import numpy as np
from sympy import re
import impactEffects.instances.ImpactorClass
from impactEffects.functions import *
from impactEffects.functions.function import *
from impactEffects.instances import ImpactorClass, TargetClass
from ans_config import *

# TODO： add the test for more cases


def test_Kinetic_energy():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=0, distance=75, density=2500)

    res = kinetic_energy(impactor)
    print(res, energy0)
    assert np.allclose(res, energy0)


def test_kinetic_energy_megatons():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=0, distance=75, density=2500)

    res = kinetic_energy_megatons(impactor)
    print(res, energy0_megatons)
    assert np.allclose(res, energy0_megatons)


def test_rec_time():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=0, distance=75, density=2500)

    res = rec_time(impactor)
    print(res, rec_time_)
    assert np.allclose(res, rec_time_)


def test_iFactor():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=0, distance=75, density=2500)

    collins_iFactor, _av, _rStrength = iFactor(impactor, targets)
    print(collins_iFactor, i_Factor)
    assert np.allclose(collins_iFactor, i_Factor)


def test_altitude_of_breakup():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=0, distance=75, density=2500)

    res = altitude_of_breakup(impactor, targets)

    print(res, altitudeBurst)
    assert np.allclose(res, altitudeBU)


def test_velocity_at_breakup():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=0, distance=75, density=2500)

    res = velocity_at_breakup(impactor, targets)

    print(res, vBU)
    assert np.allclose(res, vBU)


def test_dispersion_length_scale():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=0, distance=75, density=2500)

    res = dispersion_length_scale(impactor, targets)

    print(res, lDisper)
    assert np.allclose(res, lDisper)


def test_airburst_altitude():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=0, distance=75, density=2500)

    res = airburst_altitude(impactor, targets)

    print(res, altitudeBurst)
    assert np.allclose(res, altitudeBurst)


def test_brust_velocity():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=0, distance=75, density=2500)

    res = brust_velocity(impactor, targets)
    print(res, velocity)

    assert np.allclose(res, velocity)


def test_dispersion_of_impactor():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=0, distance=75, density=2500)

    res = dispersion_of_impactor(impactor, targets)


def test_fraction_of_momentum():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=0, distance=75, density=2500)

    res = fraction_of_momentum(impactor, targets)

    print(res, lratio, pratio)
    assert np.allclose(res[0], lratio)
    assert np.allclose(res[1], pratio)


def test_cal_trot_change():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=0, distance=75, density=2500)

    res = cal_trot_change(impactor, targets)

    assert np.allclose(res, trot_change)


def test_cal_energy_atmosphere():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=0, distance=75, density=2500)

    res = cal_energy_atmosphere(impactor, targets)

    assert np.allclose(res, energy_atmosphere)


def test_cal_energy_blast_surface():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=0, distance=75, density=2500)

    res = cal_energy_blast_surface(impactor, targets)

    assert np.allclose(res[0], energy_blast)
    assert np.allclose(res[1], energy_surface)


def test_cal_mass_of_water():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=0, distance=75, density=2500)

    res = cal_mass_of_water(impactor, targets)

    assert np.allclose(res, mwater)


def test_cal_velocity_projectile():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=0, distance=75, density=2500)

    res = cal_velocity_projectile(impactor, targets)
    print(res, vseafloor)

    assert np.allclose(res, vseafloor)


def test_cal_energy_at_seafloor():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=0, distance=75, density=2500)

    res = cal_energy_at_seafloor(impactor, targets)
    print(res, energy_seafloor)

    assert np.allclose(res, energy_seafloor)


def test_cal_ePIcentral_angle():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=0, distance=75, density=2500)

    res = cal_ePIcentral_angle(targets)
    print(res, delta)

    assert np.allclose(res, delta)


def test_cal_scaling_diameter_constant():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=0, distance=75, density=2500)

    res = cal_scaling_diameter_constant(targets)
    print(res, Cd, beta)

    assert np.allclose(res[0], Cd)
    assert np.allclose(res[1], beta)


def test_cal_anglefac():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=0, distance=75, density=2500)

    res = cal_anglefac(impactor)
    print(res, anglefac)

    assert np.allclose(res, anglefac)


def test_cal_wdiameter():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=1111, distance=75, density=1000)

    res = cal_wdiameter(impactor, targets)
    wdiameter = 873.96211031212
    print(res, wdiameter)

    assert np.allclose(res, wdiameter)


def test_cal_transient_crater_diameter():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=0, distance=75, density=2500)

    res = cal_transient_crater_diameter(impactor, targets)
    print(res, Dtr)

    assert np.allclose(res, Dtr)


def test_cal_depthr():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=0, distance=75, density=2500)

    res = cal_depthr(impactor, targets)
    print(res, depthtr)

    assert np.allclose(res, depthtr)


def test_cal_cdiamater():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=0, distance=75, density=2500)

    res = cal_cdiamater(impactor, targets)
    print(res, cdiameter)

    assert np.allclose(res, cdiameter)


def test_cal_depthfr():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=0, distance=75, density=2500)

    res = cal_depthfr(impactor, targets)
    print(res, depthfr)

    assert np.allclose(res, depthfr)


def test_cal_vCrater_vRation():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=0, distance=75, density=2500)

    res = cal_vCrater_vRation(impactor, targets)
    print(res, vCrater, vratio)

    assert np.allclose(res[0], vCrater)
    assert np.allclose(res[1], vratio)


def test_cal_vMelt():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=0, distance=75, density=2500)

    res = cal_vMelt(impactor, targets)
    print(res, vMelt)

    assert np.allclose(res, vMelt)


def test_cal_mratio_and_mcratio():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=0, distance=75, density=2500)

    res = cal_mratio_and_mcratio(impactor, targets)
    print(res, mratio, mcratio)

    assert np.allclose(res[0], mratio)
    assert np.allclose(res[1], mcratio)


def test_cal_eject_arrival():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111000, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=1111, distance=75, density=1000)

    res = cal_eject_arrival(impactor, targets)
    ejecta_arrival = 124.569530217127
    print(res, ejecta_arrival)

    assert np.allclose(res, ejecta_arrival)


def test_cal_ejecta_thickness():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111000, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=1111, distance=75, density=1000)

    res = cal_ejecta_thickness(impactor, targets)
    ejecta_thickness = 143311.274150426

    print(res, ejecta_thickness)
    assert np.allclose(res, ejecta_thickness)


def test_cal_themal():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111000, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=1111, distance=75, density=1000)

    (
        h,
        Rf,
        thermal_exposure_,
        no_radiation_,
        max_rad_time_,
        irradiation_time_,
        megaton_factor_,
        thermal_power_,
    ) = cal_themal(impactor, targets)
    (
        thermal_exposure,
        no_radiation,
        max_rad_time,
        irradiation_time,
        megaton_factor,
        thermal_power,
    ) = (
        4.14356223682368,
        0,
        14.208291667122,
        20466.8083549098,
        69.9012618159051,
        13,
    )

    assert np.allclose(thermal_exposure, thermal_exposure_)
    assert np.allclose(no_radiation, no_radiation_)
    assert np.allclose(max_rad_time, max_rad_time_)
    assert np.allclose(irradiation_time, irradiation_time_)
    assert np.allclose(megaton_factor, megaton_factor_)
    assert np.allclose(thermal_power, thermal_power_)


def test_cal_magnitude():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111000, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=1111, distance=75, density=1000)

    res = cal_magnitude(impactor, targets)
    magnitude = 11.9138097245741

    assert np.allclose(res, magnitude)


def test_cal_magnitude2():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111000, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=1111, distance=75, density=1000)

    eff_mag = 10.3894097245741
    seismic_arrival = 15
    res = cal_magnitude2(impactor, targets)

    print(res)

    assert np.allclose(res[0], eff_mag)
    assert np.allclose(res[1], seismic_arrival)


def test_cal_shock_arrival():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=0, distance=75, density=2500)

    res = cal_shock_arrival(impactor, targets)
    print(res, shock_arrival)

    assert np.allclose(res, shock_arrival)


def test_cal_vmax():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=0, distance=75, density=2500)

    res = cal_vmax(impactor, targets)
    assert np.allclose(res[0], vmax)


def test_cal_dec_level():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=0, distance=75, density=2500)

    res = cal_dec_level(impactor, targets)

    assert np.allclose(res, dec_level)


def test_cal_TsunamiArrivalTime():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=1111, distance=75, density=1000)

    res = cal_TsunamiArrivalTime(impactor, targets)
    TsunamiArrivalTime = 1436.89232538551

    print(res, TsunamiArrivalTime)
    assert np.allclose(res, TsunamiArrivalTime)


def test_cal_WaveAmplitudeUpperLimit():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=1111, distance=75, density=1000)

    res = cal_WaveAmplitudeUpperLimit(impactor, targets)
    WaveAmplitudeUpperLimit = 0.712889118910467

    print(res, WaveAmplitudeUpperLimit)
    assert np.allclose(res, WaveAmplitudeUpperLimit)


def test_cal_WaveAmplitudeLowerLimit():
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=1111, distance=75, density=1000)

    res = cal_WaveAmplitudeLowerLimit(impactor, targets)
    WaveAmplitudeLowerLimit = 0.00103553906729725

    print(res, WaveAmplitudeLowerLimit)
    assert np.allclose(res, WaveAmplitudeLowerLimit)



# -*- encoding: utf-8 -*-
"""
Filename         :Impactor.py
Description      :
Time             :2022/07/10 09:38:24
Author           :daniel
Version          :1.0
"""

from impactEffects.core.config import *


class Impactor:
    """
    __init__
    """

    def __init__(self, diameter, density, velocity, theta):
        self.pdiameter = diameter
        self.density = density
        self.velocity = velocity
        self.theta = theta
        # self.ttype = ttype

        self.mass = None
        self.energy0 = None
        self.energy0_megatons = None
        self.rec_time = None

        return

    def calculate_mass(self):
        self.mass = ((PI * self.pdiameter ** 3) / 6) * self.density

    def calculate_energy0(self):
        if self.mass is None:
            self.calculate_mass()

        self.energy0 = 0.5 * self.mass * (self.velocity * 1000) ** 2

    def calcualte_energy0_megatons(self):
        if self.energy0 is None:
            self.calculate_energy0()

        self.energy0_megatons = self.energy0 * joules2megatones

    def calculate_recTime(self):
        mass = self.get_mass()

        # Compute the recurrence interval for this energy impact
        # New model (after Bland and Artemieva (2006) MAPS 41 (607-621).
        if mass < 3:
            self.rec_time = 10 ** (-4.568) * mass ** 0.480
        elif mass < 1.7e10:
            self.rec_time = 10 ** (-4.739) * mass ** 0.926
        elif mass < 3.3e12:
            self.rec_time = 10 ** (0.922) * mass ** 0.373
        elif mass < 8.4e14:
            self.rec_time = 10 ** (-0.086) * mass ** 0.454
        else:
            self.rec_time = 10 ** (-3.352) * mass ** 0.672

        energy0_megatons = self.get_energy0_megatons()
        self.rec_time = max(self.rec_time, 110 * energy0_megatons ** 0.77)

    def get_energy0(self) -> float:
        if self.energy0 is None:
            self.calculate_energy0()
        return self.energy0

    def get_energy0_megatons(self) -> float:
        if self.energy0_megatons is None:
            self.calcualte_energy0_megatons()

        return self.energy0_megatons

    def get_rec_time(self):
        if self.rec_time is None:
            self.calculate_recTime()
        return self.rec_time

    def get_pdiameter(self) -> float:
        return self.pdiameter

    def get_density(self) -> float:
        return self.density

    def get_velocity(self) -> float:
        return self.velocity

    def get_mass(self) -> float:
        if self.mass is None:
            self.calculate_mass()

        return self.mass

    def get_theta(self):
        return self.theta


if __name__ == "__main__":
    impactor = Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    print(impactor.get_pdiameter())



# -*- encoding: utf-8 -*-
"""
Filename         :Impactor_population.py
Description      :
Time             :2022/07/10 09:38:36
Author           :daniel
Version          :1.0
"""

import random


class Distribution:
    def __init__(self):
        return


class nomal_distribution(Distribution):
    def __init__(self, mean, std):
        super().__init__()
        self.mean = mean
        self.std = std
        return

    def get_value(self):
        return random.normalvariate(self.mean, self.std)


# class Impactor_population:
#     def __init__(self):
#         return



from impactEffects.core.config import *


def print_energy(energy0, energy0_megatons):
    energy_power = log(energy0)/log(10)
    energy_power = int(energy_power)
    energy0 /= 10**energy_power

    megaton_power = log(energy0_megatons)/log(10)
    megaton_power = int(megaton_power)
    energy0_megatons /= 10**megaton_power

    # Print the energy
    if energy_power == 0:
        print("Energy before atmospheric entry: %.2f Joules " %
              (energy0),  end='')
    else:
        print("Energy before atmospheric entry: %.2f x 10^%.0f Joules " %
              (energy0, energy_power),  end='')

    if megaton_power == 0:
        if energy0_megatons < 1:
            print(" = %.2f KiloTons TNT" % (energy0_megatons * 1000),  end='')
        else:
            print(" = %.2f MegaTons TNT" % (energy0_megatons),  end='')
    else:
        print(" = %.2f x 10^%.0f MegaTons TNT" %
              (energy0_megatons, megaton_power),  end='')

    print("\n")


def print_recurrencetime(rec_time):
    # Use scientific notation for recurrence times longer than 1000 years
    if rec_time > 1000:
        rec_time_p = log(rec_time)/log(10)
        rec_time_p = int(rec_time_p)
        rec_time /= 10**rec_time_p

        if rec_time * 10**rec_time_p < 4.5e9:
            print("The average interval between impacts of this size somewhere on Earth during the last 4 billion years is %.1f x 10^%.0fyears." % (
                rec_time, rec_time_p))
        else:
            print(
                "The average interval between impacts of this size is longer than the Earth's age.")
            print("Such impacts could only occur during the accumulation of the Earth, between 4.5 and 4 billion years ago.")

        return

    # Use normal notation for intervals less than 1000 years
    if rec_time * 12 < 1:
        print("The average interval between impacts of this size somewhere on Earth is less than 1 month.")
    else:
        print("The average interval between impacts of this size somewhere on Earth is %.1f years." % (rec_time))
    print("")


def print_change(vratio, mratio, lratio, trot_change, pratio):
    print("Major Global Changes:")

    if vratio >= 0.1:
        if vratio >= 0.5:
            print("The Earth is completely disrupted by the impact and its debris forms a new asteroid belt orbiting the sun between Venus and Mars.")
        else:
            print("The Earth is strongly disturbed by the impact, but loses little mass.")
    else:
        print(
            "The Earth is not strongly disturbed by the impact and loses negligible mass.")

    if mratio >= 0.01:
        print("%.2f percent of the Earth is melted" % (mratio * 100))

    if lratio >= 0.001:
        print("Depending on the direction and location the collision, ")
        if lratio < 0.01:
            print(
                "the impact may make a very small change in the tilt of Earth's axis (< half a degree).")
        elif lratio < 0.1:
            print(
                "the impact may make a noticeable change in the tilt of Earth's axis (< 5 degrees).")
        elif lratio < 1.0:
            print("the impact may make a significant change in the tilt of Earth's axis.")
        else:
            print(
                "the impact may totally change the Earth's rotation period and the tilt of its axis.")
    else:
        print("The impact does not make a noticeable change in the tilt of Earth's axis (< 5 hundreths of a degree).")

    # Print the change in length of the day if more than one second
    if trot_change > 1e-3:
        print("Depending on the direction and location of impact, the collision may cause a change in the length of the day of up to %f unit." % (trot_change))

    if pratio >= 0.001:
        if pratio < .01:
            print("The impact shifts the Earth's orbit noticeably.")
        elif pratio < 0.1:
            print("The imapct shifts the Earth's orbit substantially.")
        else:
            print("The impact shifts the Earth's orbit totally.")

    else:
        print("The impact does not shift the Earth's orbit noticeably.")

    print("")


def print_atmospheric_entry(mass, vInput, velocity, iFactor, altitudeBU, altitudeBurst, pdensity, dispersion, theta, energy_surface, energy_megatons):
    print("Atmospheric Entry:")

    en = 0.5 * mass * ((vInput * 1000)**2 - (velocity * 1000)**2)

    en_mton = en / (4.186 * 10**15)  # joules to megatons conversion

    en_power = log(en)/log(10)
    en_power = int(en_power)
    en /= 10**en_power

    ens_power = log(energy_surface)/log(10)
    ens_power = int(ens_power)
    energy_surface /= 10**ens_power

    enmton_power = log(en_mton)/log(10)
    enmton_power = int(enmton_power)
    en_mton /= 10**enmton_power

    megaton_power = log(energy_megatons)/log(10)
    megaton_power = int(megaton_power)
    energy_megatons /= 10**megaton_power

    if iFactor >= 1:
        print("The projectile lands intact, with a velocity %f km/s = %f miles/s." %
              (velocity, velocity * 0.621))
        print("The energy lost in the atmosphere is %.2f x 10^%.0f Joules = %.2f x 10^%.0f MegaTons." % (
            en, en_power, en_mton, enmton_power))
    else:
        print("The projectile begins to breakup at an altitude of %f meters = %f ft" %
              (altitudeBU, altitudeBU * 3.28))
        if altitudeBurst > 0:
            print("The projectile bursts into a cloud of fragments at an altitude of %f meters = %f ft" %
                  (altitudeBurst, altitudeBurst * 3.28))
            print("The residual velocity of the projectile fragments after the burst is %f km/s = %f miles/s" %
                  (velocity, velocity * 0.621))
            print("The energy of the airburst is %.2f x 10^%.0f Joules = %.2f x 10^%.0f MegaTons." %
                  (en, en_power, en_mton, enmton_power))
            if pdensity < 5000:
                print(
                    "No crater is formed, although large fragments may strike the surface.")
            else:
                print("Large fragments strike the surface and may create a crater strewn field.  A more careful treatment of atmospheric entry is required to accurately estimate the size-frequency distribution of meteoroid fragments and predict the number and size of craters formed.")

        else:
            print("The projectile reaches the ground in a broken condition.  The mass of projectile strikes the surface at velocity %f km/s = %f miles/s" %
                  (velocity, velocity * 0.621))
            print("The energy lost in the atmosphere is %.2f x 10^%.0f Joules = %.2f x 10^%.0f MegaTons." %
                  (en, en_power, en_mton, enmton_power))
            if megaton_power != 0:
                print("The impact energy is %.2f x 10^%.0f Joules = %.2f x 10^%.0fMegaTons." %
                      (energy_surface, ens_power, energy_megatons, megaton_power))
            else:
                print("The impact energy is %.2f x 10^%.0f Joules = %.2f MegaTons." %
                      (energy_surface, ens_power, energy_megatons))

            print(
                "The larger of these two energies is used to estimate the airblast damage.")
            print("The broken projectile fragments strike the ground in an ellipse of dimension %f km by %f km" %
                  (dispersion/(1000 * sin(theta * PI / 180)), dispersion / 1000))

    print("")


def print_ejecta(energy_megatons, megaton_power, distance, Rf, Dtr, cdiameter, ejecta_arrival, ejecta_thickness, d_frag):
    # ejecta results
    print("Ejecta:")

    # Ejecta from small impacts is blocked by the atmosphere
    if (energy_megatons * 10**megaton_power) < 200 and (distance > Rf):
        print("Most ejecta is blocked by Earth's atmosphere")
        return

    # Ejecta comes from transient crater
    if distance * 1000 <= Dtr/2:
        print("Your position was inside the transient crater and ejected upon impact")
        return

    # Inside final crater
    if distance * 1000 > Dtr/2 and distance * 1000 < cdiameter/2:
        print("Your position is in the region which collapses into the final crater.")
        return

    # Arrival time greater than 1 hour or almost no ejecta
    if ejecta_arrival >= 3600:
        print("Little rocky ejecta reaches this site fallout is dominated by condensed vapor from the projectile.")
        return
    elif ejecta_thickness * 10**6 < 1:
        print("Almost no solid ejecta reaches this site.")
        return

    # Type of ejecta deposit
    if distance * 1000 <= 3 * cdiameter/2:
        print("Your position is beneath the continuous ejecta deposit.")
    else:
        print("At your position there is a fine dusting of ejecta with occasional larger fragments")

    # Ejecta thickness
    print("Average Ejecta Thickness:  %f %s ( = %f %s ) \n" % (
          ejecta_thickness, "meters", ejecta_thickness * 3.28, "feet"))

    # Fragment size
    print("Mean Fragment Diameter:  %f %s ( = %f %s ) \n" % (
          d_frag, "meters", d_frag * 3.28, "feet"))
    print("")


def print_thermal(velocity, no_radiation, max_rad_time, distance, Rf, h, thermal_power, thermal_exposure, irradiation_time):
    print("Thermal Radiation:")

    print("What does this mean?")

    # No fireball at low velocity
    if velocity < 15:
        print("At this impact velocity ( < 15 km/s), little vaporization occurs no fireball is created, therefore, there is no thermal radiation damage.")
        return

    # Is fireball above the horizon?
    if no_radiation == 1:
        print(
            "The fireball is below the horizon. There is no direct thermal radiation.\n</dl>\n")
        return

    # Time of maximum radiation
    print("Time for maximum radiation:  %f %s  after impact" %
          (max_rad_time, "seconds"))

    # Size of the fireball
    if distance < Rf:
        print("Your position is inside the fireball.")
    else:
        print("Visible fireball radius:  %f %s ( = %f %s ) " %
              ((Rf-h)*1000, "meters", (Rf-h)*1000*3.28, "feet"))

    # Brightness of the fireball relative to the sun
    B = (Rf - h)/(4.4 * 10**-3 * distance)
    if B >= 0.1:
        print("The fireball appears %f times larger than the sun" % (B))

    # Thermal exposure
    if thermal_power == 0:
        print("Thermal Exposure:  %.2f Joules/m<sup2</sup>" % (
              thermal_exposure))
    else:
        print("Thermal Exposure:  %.2f x 10<sup>%.0f</sup> Joules/m<sup>2</sup>" %
              (thermal_exposure, thermal_power))

    # Duration of irradiation
    print("Duration of Irradiation:  %f %s" % (
          irradiation_time, "seconds"))

    # Radiant flux relative to solar flux
    flux = (thermal_exposure * 10**thermal_power) / (irradiation_time * 1000)
    print("Radiant flux (relative to the sun):  %f", flux)
    if flux >= 15 and flux <= 25:
        print(" (Flux from a burner on full at a distance of 10 cm)")
    print("")


def print_seismic(magnitude, seismic_arrival):
    # seismic results
    print("<dl>\n<dt>\n<h2>Seismic Effects:</h2>\n")
    print("What does this mean?")

    # Don' print results if magnitude very small
    if magnitude < 0:
        print("The Richter Scale Magnitude for this impact is less than zero no seismic shaking will be felt.")
        return

    # Arrival time
    if seismic_arrival < 0.1:
        print("The major seismic shaking will arrive almost instantly.")
    else:
        print("The major seismic shaking will arrive approximately %f %s after impact." %
              (seismic_arrival, "seconds"))

    # Richter Scale Magnitude
    print("Richter Scale Magnitude:  %.1f" % (magnitude))
    if magnitude >= 9.5:
        print(" (This is greater than any earthquake in recorded history)")

    # Earthquake damage
    # print("Mercalli Scale Intensity at a distance of distance km: \n<br>\n"
    # print des
    print("")


def print_airblast(opressure, vmax, shock_arrival, distance, altitudeBurst, dec_level, shock_damage):
    bars = opressure / 10**5
    mph = vmax * 2.23694

    print("Air Blast:")

    print("What does this mean?")

    # Exit if airblast has no effect
    if opressure < 1:
        print("The air blast at this location would not be noticed. (The overpressure is less than 1 Pa)")
        return

    # Blast wave arrival time
    print("The air blast will arrive approximately %f %s after impact." %
          (shock_arrival, "seconds"))

    # Overpressure
    if distance*1000. < 3*altitudeBurst:
        print("Peak Overpressure:  %f - %f Pa = %f - %f bars = %f - %f psi" %
              (opressure, 2*opressure, bars, 2*bars, bars * 14.2, 2*bars * 14.2))
    else:
        print("Peak Overpressure:  %f Pa = %f bars = %f psi" %
              (opressure, bars, bars * 14.2))

    # Wind velocity
    print("Max wind velocity:  %f m/s = %f mph" % (vmax, mph))

    # Sound intensity
    if dec_level > 0:
        print("Sound Intensity:  %.0f dB" % (dec_level), end="")
        if dec_level <= 20:
            print(" (Barely Audible)")
        elif dec_level <= 50:
            print(" (Easily Heard)")
        elif dec_level <= 90:
            print(" (Loud as heavy traffic)")
        elif dec_level <= 120:
            print(" (May cause ear pain)")
        else:
            print(" (Dangerously Loud)")

    else:
        print("The blast wave will not be heard.")

    # Airblast damage
    if shock_damage != "":
        print("Damage Description: ", end="")

    print(shock_damage)
    print("")


def print_tsunami(distance, wdiameter, TsunamiArrivalTime, WaveAmplitudeLowerLimit, WaveAmplitudeUpperLimit):
    FormattedTime = 0.
    TimeUnit = "non"

    # Print the tsunami results
    print("Tsunami Wave:")

    print("What does this mean?")

    # If inside the water crater say this and finish.
    if distance*1000 < wdiameter:
        print("Your location is within the crater formed in the water layer. This is where the impact tsunami wave is generated. \n")
        return

    # Report the approximate arrival time of the tsunami
    print("The impact-generated tsunami wave arrives approximately %.1f %s after impact. " % (
          TsunamiArrivalTime, "seconds"))

    # Report the two wave amplitude limits if more than 10 cm
    if WaveAmplitudeLowerLimit > 0.1:
        print("Tsunami wave amplitude is between:  %.1f %s ( = %.1f %s) and %.1f %s ( = %.1f %s). " % (
              WaveAmplitudeLowerLimit, "meters", WaveAmplitudeLowerLimit*3.28, "feets", WaveAmplitudeUpperLimit, "meters", WaveAmplitudeUpperLimit*3.28, "feets"))
    elif WaveAmplitudeUpperLimit < 0.1:
        print(
            "Tsunami wave amplitude is less than 10 cm at your location. ")
    else:
        print("Tsunami wave amplitude is less than %.1f %s ( = %.1f %s). " % (
              WaveAmplitudeUpperLimit, "meters",  WaveAmplitudeUpperLimit*3.28, "feets"))

    print("")


if __name__ == "__main__":
    print_energy(4.8967338468637005e+17, 116.9788305509723)



# -*- encoding: utf-8 -*-
"""
Filename         :impact.py
Description      :
Time             :2022/07/10 10:39:57
Author           :daniel
Version          :1.0
"""
from impactEffects.functions.function import *


def atmospheric_entry(impactor: Impactor, target: Target):
    """

    Arguments
    ---------


    Returns
    -------

    """

    i_factor, _av, _rStrength = iFactor(impactor, target)

    if i_factor >= 1:
        velocity_at_surface = burst_velocity_at_zero(impactor, target)
    else:
        # Compute the breakup altitude by
        # combining above parameters to evaluate Eq. 11
        altitudeBU = altitude_of_breakup(
            target.scaleHeight, _rStrength, i_factor
        )

        # Define velocity at breakup
        # altitude using Eq. 8 (and Eq. 5)
        vBU = velocity_at_breakup(
            impactor.get_velocity(),
            _av,
            altitudeBU,
            target.get_schaleHeight(),
        )

        # Define dispersion length-scale (Eq. 16)
        lDisper = dispersion_length_scale(
            impactor.get_pdiameter(),
            impactor.get_theta(),
            impactor.get_density(),
            target.get_dragC(),
            target.get_rhoSurface(),
            altitudeBU,
            target.get_schaleHeight(),
        )

        # Define the alpha parameters
        # used to evaluate Eq. 18 and Eq. 19
        alpha2 = (target.get_fp() ** 2 - 1) ** (1 / 2)

        # Define the burst altitude using Eq. 18
        altitudeBurst = airburst_altitude(
            impactor, target, alpha2, lDisper, altitudeBU
        )

        velocity_at_surface = brust_velocity(
            impactor, target, altitudeBurst, altitudeBU, vBU, lDisper
        )
        dispersion = dispersion_of_impactor(
            impactor, target, lDisper, altitudeBU, altitudeBurst
        )

    return (
        velocity_at_surface,
        i_factor,
        altitudeBU,
        altitudeBurst,
        dispersion,
    )


def calc_energy(impactor: Impactor, target: Target):
    """
    Implement equation (1)
    """
    mass, energy0, energy0_megatons, theta, R_earth = (
        impactor.get_mass(),
        impactor.get_energy0(),
        impactor.get_energy0_megatons(),
    )

    # If the impactor is less than a kilogram,
    # the impactor burns up in the atmosphere
    if mass < 1:
        logging.warning(
            "Impactor is less than a kilogram. \
                Impactor will burn up in the atmosphere."
        )

    # Calculate the effects of atmospheric entry
    (
        velocity,
        iFactor,
        altitudeBU,
        altitudeBurst,
        dispersion,
    ) = atmospheric_entry(impactor, target)

    # Compute linear and angular momentum as a fraction of Earth's
    linmom, angmom, energy0 = fraction_of_momentum(
        impactor, target, velocity
    )

    trot_change = cal_trot_change(impactor, target, velocity)

    # Compute energy of airburst, or energy after deceleration by atmosphere
    energy_atmosphere = cal_energy_atmosphere(impactor, target, velocity)
    energy_blast, energy_surface = cal_energy_blast_surface(
        impactor, target, velocity, altitudeBurst, energy_atmosphere
    )
    energy_megatons = energy_surface / (
        4.186 * 10 ** 15
    )  # joules to megatons conversion

    # Account for the decelerating effect of the water layer
    mwater = cal_mass_of_water(impactor, target, velocity)
    vseafloor = cal_velocity_projectile(impactor, target, velocity)
    energy_seafloor = cal_energy_at_seafloor(impactor, target, vseafloor)

    # Compute the epicentral angle for
    # use in several subsequent calculations.
    delta = cal_energy_at_seafloor(target)

    return (
        velocity,
        altitudeBU,
        altitudeBurst,
        dispersion,
        linmom,
        angmom,
        energy0,
        trot_change,
        energy_atmosphere,
        energy_blast,
        energy_surface,
        energy_megatons,
        mwater,
        vseafloor,
        energy_seafloor,
        delta,
    )


def air_blast(
    impactor: Impactor,
    target: Target,
    energy_blast: float = None,
    altitudeBurst: float = None,
):
    Po = target.get_Po()
    vsound = 330  # speed of sound in m/s
    # radius at which relationship
    # between overpressure and distance changes
    r_cross = 0
    # radius at which relationship between
    # overpressure and distance changes (for surface burst)
    r_cross0 = 290
    op_cross = 75000  # overpressure at crossover
    energy_ktons = 0  # energy in kilotons
    d_scale = 0  # distance scaled for 1 kTon blast
    slantRange = 0  # in km
    d_smooth = 0
    p_machT = 0
    p_regT = 0

    # energy_ktons = 1000 * energy_megatons
    energy_ktons = energy_blast

    # Arrival time is straight line distance divided by sound speed
    # for air burst, distance is slant range from explosion
    slantRange = (
        target.get_distance() ** 2 + (altitudeBurst / 1000) ** 2
    ) ** (1 / 2)
    # distance in meters divided by velocity of sound in m/s
    shock_arrival = (slantRange * 1000) / vsound

    # Scale distance to equivalent for a kiloton explosion
    sf = (energy_ktons) ** (1 / 3)
    d_scale = (target.get_distance() * 1000) / sf

    # Scale burst altitude to equivalent for a kiloton explosion
    z_scale = altitudeBurst / sf
    r_cross = r_cross0 + 0.65 * z_scale
    r_mach = 550 * z_scale / (1.2 * (550 - z_scale))
    if z_scale >= 550:
        r_mach = 1e30

    if altitudeBurst > 0:
        d_smooth = z_scale ** 2 * 0.00328
        p_machT = (
            ((r_cross * op_cross) / 4)
            * (1 / (r_mach + d_smooth))
            * (1 + 3 * (r_cross / (r_mach + d_smooth)) ** (1.3))
        )
        p_regT = 3.14e11 * ((r_mach - d_smooth) ** 2 + z_scale ** 2) ** (
            -1.3
        ) + 1.8e7 * ((r_mach - d_smooth) ** 2 + z_scale ** 2) ** (-0.565)
    else:
        d_smooth = 0
        p_machT = 0

    if d_scale >= (r_mach + d_smooth):
        opressure = (
            ((r_cross * op_cross) / 4)
            * (1 / d_scale)
            * (1 + 3 * (r_cross / d_scale) ** (1.3))
        )
    elif d_scale <= (r_mach - d_smooth):
        opressure = 3.14e11 * (d_scale ** 2 + z_scale ** 2) ** (
            -1.3
        ) + 1.8e7 * (d_scale ** 2 + z_scale ** 2) ** (-0.565)
    else:
        opressure = (
            p_regT
            - (d_scale - r_mach + d_smooth)
            * 0.5
            * (p_regT - p_machT)
            / d_smooth
        )

    # Wind velocity
    vmax = ((5 * opressure) / (7 * Po)) * (
        vsound / (1 + (6 * opressure) / (7 * Po)) ** (1 / 2)
    )

    # sound intensity
    if opressure > 0:
        dec_level = 20 * (log(opressure) / log(10))
    else:
        dec_level = 0

    return shock_arrival, vmax, dec_level


def tsunami(
    impactor: Impactor, target: Target, wdiameter: float = None
) -> float:
    shallowness = 0  # Ratio of Impactor diameter to water depth
    MaxWaveAmplitude = 0  # Maximum rim wave amplitude
    MaxWaveRadius = (
        0  # Radius where max rim wave is formed (upper estimate)
    )
    MinWaveRadius = (
        0  # Radius where max rim wave is formed (lower estimate)
    )
    CollapseWaveRadius = 0  # Radius where collapse wave is formed
    RimWaveExponent = 0  # Attenuation factor for rim wave
    CollapseWaveExponent = 0  # Attenuation factor for collapse wave
    MaxCollapseWaveAmplitude = 0  # Maximum collapse wave amplitude
    CollapseWaveAmplitude = (
        0  # Amplitude of collapse wave at specified distance
    )
    TsunamiSpeed = 0  # Tsunami speed in m/s
    TsunamiWavelength = 0  # Tsunami wavelength in m

    # Define parameters
    shallowness = impactor.get_pdiameter() / target.get_depth()
    RimWaveExponent = 1.0
    MaxWaveRadius = 0.001 * wdiameter
    MinWaveRadius = 0.0005 * wdiameter

    # Tsunami arrival time assumes linear wave theory
    TsunamiWavelength = 2.0 * wdiameter
    TsunamiSpeed = sqrt(
        0.5
        * target.get_g()
        * TsunamiWavelength
        / PI
        * tanh(2.0 * PI * target.get_depth() / TsunamiWavelength)
    )
    TsunamiArrivalTime = target.get_distance() * 1000 / TsunamiSpeed

    # Rim wave upper and lower limit estimates
    MaxWaveAmplitude = min(0.07 * wdiameter, target.get_depth())
    WaveAmplitudeUpperLimit = (
        MaxWaveAmplitude
        * (MaxWaveRadius / target.get_distance()) ** RimWaveExponent
    )
    WaveAmplitudeLowerLimit = (
        MaxWaveAmplitude
        * (MinWaveRadius / target.get_distance()) ** RimWaveExponent
    )

    # Collapse wave correction to lower limit for deep-water impacts
    if shallowness < 0.5:
        CollapseWaveExponent = 3.0 * exp(-0.8 * shallowness)
        CollapseWaveRadius = 0.0025 * wdiameter
        MaxCollapseWaveAmplitude = 0.06 * min(
            wdiameter / 2.828, target.get_depth()
        )
        CollapseWaveAmplitude = (
            MaxCollapseWaveAmplitude
            * (CollapseWaveRadius / target.get_distance())
            ** CollapseWaveExponent
        )
        WaveAmplitudeLowerLimit = min(
            CollapseWaveAmplitude, WaveAmplitudeLowerLimit
        )

    return (
        TsunamiArrivalTime,
        WaveAmplitudeUpperLimit,
        WaveAmplitudeLowerLimit,
    )



from tkinter.tix import InputOnly
import numpy as np
from sqlalchemy import false
from sympy import re
import impactEffects.instances.ImpactorClass
from impactEffects.functions import *
from impactEffects.functions.function import *
from impactEffects.instances import ImpactorClass, TargetClass
from impactEffects.utils.print import print_airblast, print_change, print_ejecta, print_energy, print_recurrencetime, print_atmospheric_entry, print_seismic, print_thermal, print_tsunami


def get_input():
    input_fail = True 
    while True:
        pdiameter = float(input("please input diameter of impactor: "))
        pdensity = float(input("please input density of impactor: "))
        v_input = float(input("please input the velocity of impactor: "))
        theta = float(input("please input the theta of impactor: "))
        tdensity = float(input("please input the density of target: "))
        depth = float(input("please input the depth(meters): "))
        distance = float(input("please input the distance: "))
        
        try:
            impactor = impactEffects.instances.ImpactorClass.Impactor(
                diameter=pdiameter, density=pdensity, velocity= v_input, theta=theta
            )
            targets = TargetClass.Target(depth=depth, distance=distance, density=tdensity)
            break
        except:
            print("Input error, please retry or exit.")
            retry = input("Do you want to retry?(0:false, 1:true)")
            if retry == "0":
                exit(0)
            elif retry == "1":
                continue
            else:
                print("error: input is not 0/1 .")
                exit(1)

    return impactor, targets

def simulateImpactor(impartor: Impactor, targets: Target):
    
    # cal_energy
    _kinetic_energy = kinetic_energy(impactor)
    _kinetic_energy_megatons = kinetic_energy_megatons(impactor)
    print_energy(_kinetic_energy, _kinetic_energy_megatons)

    _rec_time = rec_time(impactor)
    print_recurrencetime(_rec_time)

    ## atmospheric_entry
    collins_iFactor, _av, _rStrength = iFactor(impactor, targets)
    
    if collins_iFactor >= 1:
        velocity = burst_velocity_at_zero(impactor, targets)
    else:
        altitudeBU = altitude_of_breakup(impactor, targets)
        vBU = velocity_at_breakup(impactor, targets)

        lDisper = dispersion_length_scale(impactor, targets)

        altitudeBurst = airburst_altitude(impactor, targets)

        velocity = brust_velocity(impactor, targets)

        dispersion = dispersion_of_impactor(impactor, targets)

    lratio, pratio = fraction_of_momentum(impactor, targets)

    trot_change = cal_trot_change(impactor, targets)
    energy_atmosphere = cal_energy_atmosphere(impactor, targets)
    energy_blast, energy_surface = cal_energy_blast_surface(impactor, targets)
    
    
    mwater = cal_mass_of_water(impactor, targets)
    vseafloor = cal_velocity_projectile(impactor, targets)
    energy_seafloor = cal_energy_at_seafloor(impactor, targets)
    delta = cal_ePIcentral_angle(targets)
    ## end_cal_energy

    # find_crater
    Cd, beta = cal_scaling_diameter_constant(target=targets)
    anglefac = cal_anglefac(impactor)

    if targets.depth != 0:    
        wdiameter = cal_wdiameter(impactor, targets)

    Dtr = cal_transient_crater_diameter(impactor, targets)

    depthtr = cal_depthr(impactor, targets)

    cdiameter = cal_cdiamater(impactor, targets)

    depthfr = cal_depthfr(impactor, targets)

    vCrater, vRatio = cal_vCrater_vRation(impactor, targets)

    vMelt = cal_vMelt(impactor, targets)

    mratio, mcratio = cal_mratio_and_mcratio(impactor, targets)
    # end find_crater
    
    print_change(vRatio, mratio, lratio, trot_change, pratio)
    
    energy_megatons = energy_surface / (4.186 * 10**15)
    if impactor.get_mass() <= 1.5707963e12:
        print_atmospheric_entry(impactor.get_mass(), impactor.velocity, velocity, collins_iFactor, altitudeBU,
                                altitudeBurst, impactor.density, dispersion, impactor.theta, energy_surface, energy_megatons)
        
    if altitudeBurst <= 0:
        ejecta_arrival = cal_eject_arrival(impactor, targets)

        ejecta_thickness = cal_ejecta_thickness(impactor, targets)
        d_frag = cal_d_frag(impactor = impactor, target=targets, cdiameter=cdiameter, altitudeBurst = altitudeBurst, Dtr = Dtr)
        
        if velocity >= 15:
            (
                h, 
                Rf,
                thermal_exposure_,
                no_radiation_,
                max_rad_time_,
                irradiation_time_,
                megaton_factor_,
                thermal_power_,
            ) = cal_themal(impactor, targets)


        magnitude = cal_magnitude(impactor, targets)

        eff_mag, seismic_arrival = cal_magnitude2(impactor, targets)
        
        if targets.distance * 1000 <= Dtr/2:
            print_ejecta(energy_megatons, int(log(energy_megatons)/log(10)), targets.distance, 
                         Rf, Dtr, cdiameter, ejecta_arrival, ejecta_thickness, d_frag)
            return
        
        print_thermal(velocity, no_radiation_, max_rad_time_, targets.distance, Rf, 
                      h, thermal_power_, thermal_exposure_, irradiation_time_)
        print_seismic(magnitude, seismic_arrival)
        print_ejecta(energy_megatons, int(log(energy_megatons)/log(10)), targets.distance, 
                     Rf, Dtr, cdiameter, ejecta_arrival, ejecta_thickness, d_frag)
    
    # Compute the effects of the airblast and print
    shock_arrival = cal_shock_arrival(impactor, targets)
    vmax, opressure = cal_vmax(impactor, targets)
    shock_damage = cal_shock_damage(impactor=impactor, target=targets, opressure=opressure, vmax=vmax)
    dec_level = cal_dec_level(impactor, targets)
    print_airblast(opressure, vmax, shock_arrival, targets.distance, altitudeBurst, dec_level, shock_damage)
    
    # Compute the tsunami amplitude if water layer present
    if targets.depth > 0:
        TsunamiArrivalTime = cal_TsunamiArrivalTime(impactor, targets)
        WaveAmplitudeUpperLimit = cal_WaveAmplitudeUpperLimit(impactor, targets)
        WaveAmplitudeLowerLimit = cal_WaveAmplitudeLowerLimit(impactor, targets)
        print_tsunami(targets.distance, wdiameter, TsunamiArrivalTime, WaveAmplitudeLowerLimit, WaveAmplitudeUpperLimit)


if __name__ == "__main__":
    # impactor, target = get_input()
    # print(impactor.get_density())
    impactor = impactEffects.instances.ImpactorClass.Impactor(
        diameter=111, density=111, velocity=111, theta=45
    )
    targets = TargetClass.Target(depth=100, distance=75, density=1000)
    simulateImpactor(impactor, targets)
    


# -*- encoding: utf-8 -*-
"""
Filename         :core.py
Description      :
Time             :2022/07/10 09:37:51
Author           :daniel
Version          :1.0
"""

from impactEffects.instances.ImpactorClass import *
from impactEffects.instances.TargetClass import *
from impactEffects.core.config import *
from impactEffects.core.core_collins import *


def kinetic_energy(
    impactor: Impactor, type: Choices = Choices.Collins
) -> float:
    """

    Arguments
    ---------
    impactor: Instance of Impactor, containning

    Returns
    -------

    """
    if type is Choices.Collins:
        return collins_kinetic_energy(impactor)

    return None


def kinetic_energy_megatons(
    impactor: Impactor, type: Choices = Choices.Collins
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if type is Choices.Collins:
        return collins_kinetic_energy_megatons(impactor)

    return None


def rec_time(impactor: Impactor, type: Choices = Choices.Collins) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """

    if type is Choices.Collins:
        return collins_rec_time(impactor)

    return None


def iFactor(
    impactor: Impactor, target: Target, type: Choices = Choices.Collins
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """

    if type is Choices.Collins:
        return collins_cal_iFactor(impactor, target)

    return None


def burst_velocity_at_zero(
    impactor: Impactor, target: Target, type: Choices = Choices.Collins
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if type is Choices.Collins:
        return collins_burst_velocity_at_zero(impactor, target)

    return None


def altitude_of_breakup(
    impactor: Impactor,
    target: Target,
    collins_iFactor: float = None,
    type: Choices = Choices.Collins,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """

    if type is Choices.Collins:
        return collins_altitude_of_breakup(
            impactor, target, collins_iFactor
        )

    return None


def velocity_at_breakup(
    impactor: Impactor,
    target: Target,
    av: float = None,
    altitudeBU: float = None,
    type: Choices = Choices.Collins,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """

    if type is Choices.Collins:
        return collins_velocity_at_breakup(
            impactor, target, av, altitudeBU
        )

    return None


def dispersion_length_scale(
    impactor: Impactor,
    target: Target,
    altitudeBU: float = None,
    type: Choices = Choices.Collins,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if type is Choices.Collins:
        return collins_dispersion_length_scale(
            impactor, target, altitudeBU
        )

    return None


def airburst_altitude(
    impactor: Impactor,
    target: Target,
    alpha2: float = None,
    lDisper: float = None,
    altitudeBU: float = None,
    type: Choices = Choices.Collins,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if type is Choices.Collins:
        return collins_airburst_altitude(
            impactor, target, alpha2, lDisper, altitudeBU
        )

    return None


def brust_velocity(
    impactor: Impactor,
    target: Target,
    altitudeBurst: float = None,
    altitudeBU: float = None,
    vBu: float = None,
    lDisper: float = None,
    type: Choices = Choices.Collins,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """

    if type is Choices.Collins:
        return collins_brust_velocity(
            impactor, target, altitudeBurst, altitudeBU, vBu, lDisper
        )

    return None


def dispersion_of_impactor(
    impactor: Impactor,
    target: Target,
    l_disper: float = None,
    altitude_bu: float = None,
    altitude_burst: float = None,
    type: Choices = Choices.Collins,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """

    if type is Choices.Collins:
        return collins_dispersion_of_impactor(
            impactor, target, l_disper, altitude_bu, altitude_burst
        )

    return None


def fraction_of_momentum(
    impactor: Impactor,
    target: Target,
    velocity: float = None,
    type: Choices = Choices.Collins,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """

    if type is Choices.Collins:
        return collins_fraction_of_momentum(impactor, target, velocity)

    return None


def cal_trot_change(
    impactor: Impactor,
    target: Target,
    velocity: float = None,
    type: Choices = Choices.Collins,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """

    if type is Choices.Collins:
        return collins_cal_trot_change(impactor, target, velocity)

    return None


def cal_energy_atmosphere(
    impactor: Impactor,
    target: Target,
    velocity: float = None,
    type: Choices = Choices.Collins,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """

    if type is Choices.Collins:
        return collins_cal_energy_atmosphere(impactor, target, velocity)

    return None


def cal_energy_blast_surface(
    impactor: Impactor,
    target: Target,
    velocity: float = None,
    altitudeBurst: float = None,
    energy_atmosphere: float = None,
    type: Choices = Choices.Collins,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if type is Choices.Collins:
        return collins_cal_energy_blast_surface(
            impactor, target, velocity, altitudeBurst, energy_atmosphere
        )

    return None


def cal_mass_of_water(
    impactor: Impactor, target: Target, type: Choices = Choices.Collins
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """

    if type is Choices.Collins:
        return collins_cal_mass_of_water(impactor, target)

    return None


def cal_velocity_projectile(
    impactor: Impactor,
    target: Target,
    velocity: float = None,
    type: Choices = Choices.Collins,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """

    if type is Choices.Collins:
        return collins_cal_velocity_projectile(impactor, target, velocity)

    return None


def cal_energy_at_seafloor(
    impactor: Impactor,
    target: Target,
    vseafloor: float = None,
    type: Choices = Choices.Collins,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if type is Choices.Collins:
        return collins_cal_energy_at_seafloor(impactor, target, vseafloor)

    return None


def cal_ePIcentral_angle(
    target: Target, type: Choices = Choices.Collins
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """

    if type is Choices.Collins:
        return collins_cal_ePIcentral_angle(target)

    return None


def cal_scaling_diameter_constant(
    target: Target, type: Choices = Choices.Collins
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """

    if type is Choices.Collins:
        return collins_cal_scaling_diameter_constant(target=target)

    return None


def cal_anglefac(
    impactor: Impactor, type: Choices = Choices.Collins
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """

    if type is Choices.Collins:
        return collins_cal_anglefac(impactor)

    return None


def cal_wdiameter(
    impactor: Impactor,
    target: Target,
    anglefac: float = None,
    velocity: float = None,
    type: Choices = Choices.Collins,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if type is Choices.Collins:
        return collins_cal_wdiameter(impactor, target, anglefac, velocity)

    return None


def cal_transient_crater_diameter(
    impactor: Impactor,
    target: Target,
    Cd: float = None,
    beta: float = None,
    anglefac: float = None,
    vseafloor: float = None,
    type: Choices = Choices.Collins,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if type is Choices.Collins:
        return collins_cal_transient_crater_diameter(
            impactor, target, Cd, beta, anglefac, vseafloor
        )

    return None


def cal_depthr(
    impactor: Impactor,
    target: Target,
    Dtr: float = None,
    type: Choices = Choices.Collins,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if type is Choices.Collins:
        return collins_cal_depthr(impactor, target, Dtr)

    return None


def cal_cdiamater(
    impactor: Impactor,
    target: Target,
    Dtr: float = None,
    type: Choices = Choices.Collins,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if type is Choices.Collins:
        return collins_cal_cdiamater(impactor, target, Dtr)

    return None


def cal_depthfr(
    impactor: Impactor,
    target: Target,
    Dtr: float = None,
    depthtr: float = None,
    cdiameter: float = None,
    type: Choices = Choices.Collins,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if type is Choices.Collins:
        return collins_cal_depthfr(
            impactor, target, Dtr, depthtr, cdiameter
        )

    return None


def cal_vCrater(
    impactor: Impactor,
    target: Target,
    Dtr: float = None,
    type: Choices = Choices.Collins,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if type is Choices.Collins:
        return collins_cal_vCrater(impactor, target, Dtr)

    return None


def cal_vratio(
    impactor: Impactor,
    target: Target,
    vCrater: float = None,
    Dtr: float = None,
    type: Choices = Choices.Collins,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if type is Choices.Collins:
        return collins_cal_vratio(impactor, target, vCrater, Dtr)

    return None


def cal_vCrater_vRation(
    impactor: Impactor,
    target: Target,
    Dtr: float = None,
    type: Choices = Choices.Collins,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if type is Choices.Collins:
        return collins_cal_vCrater_vRation(impactor, target, Dtr)

    return None


def cal_vMelt(
    impactor: Impactor,
    target: Target,
    velocity: float = None,
    energy_seafloor: float = None,
    type: Choices = Choices.Collins,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if type is Choices.Collins:
        return collins_cal_vMelt(
            impactor, target, velocity, energy_seafloor
        )

    return None


def cal_mratio_and_mcratio(
    impactor: Impactor,
    target: Target,
    velocity: float = None,
    vMelt: float = None,
    vCrater: float = None,
    Dtr: float = None,
    type: Choices = Choices.Collins,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if type is Choices.Collins:
        return collins_cal_mratio_and_mcratio(
            impactor, target, velocity, vMelt, vCrater, Dtr
        )

    return None


def cal_eject_arrival(
    impactor: Impactor,
    target: Target,
    altitudeBurst: float = None,
    type: Choices = Choices.Collins,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if type is Choices.Collins:
        return collins_cal_eject_arrival(impactor, target, altitudeBurst)

    return None


def cal_ejecta_thickness(
    impactor: Impactor,
    target: Target,
    altitudeBurst: float = None,
    Dtr: float = None,
    type: Choices = Choices.Collins,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if type is Choices.Collins:
        return collins_cal_ejecta_thickness(
            impactor, target, altitudeBurst, Dtr
        )

    return None


def cal_d_frag(impactor: Impactor, target: Target, cdiameter: float = None, altitudeBurst: float = None, Dtr: float = None, type: Choices = Choices.Collins) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if type is Choices.Collins:
        return collins_cal_d_frag(impactor, target, cdiameter, altitudeBurst, Dtr)

    return None


def cal_themal(
    impactor: Impactor,
    target: Target,
    energy_surface: float = None,
    altitudeBurst: float = None,
    delta: float = None,
    velocity: float = None,
    type: Choices = Choices.Collins,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if type is Choices.Collins:
        return collins_cal_themal(
            impactor,
            target,
            energy_surface,
            altitudeBurst,
            delta,
            velocity,
        )

    return None


def cal_magnitude(
    impactor: Impactor,
    target: Target,
    altitudeBurst: float = None,
    energy_seafloor: float = None,
    type: Choices = Choices.Collins,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if type is Choices.Collins:
        return collins_cal_magnitude(
            impactor, target, altitudeBurst, energy_seafloor
        )

    return None


def cal_magnitude2(
    impactor: Impactor,
    target: Target,
    energy_seafloor: float = None,
    altitudeBurst: float = None,
    distance: float = None,
    surface_wave_v: float = None,
    delta: float = None,
    type: Choices = Choices.Collins,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if type is Choices.Collins:
        return collins_cal_magnitude2(
            impactor, target, altitudeBurst, energy_seafloor, delta
        )

    return None


def cal_shock_arrival(
    impactor: Impactor,
    target: Target,
    altitudeBurst: float = None,
    type: Choices = Choices.Collins,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if type is Choices.Collins:
        return collins_cal_shock_arrival(impactor, target, altitudeBurst)

    return None


def cal_vmax(
    impactor: Impactor,
    target: Target,
    energy_blast: float = None,
    altitudeBurst: float = None,
    type: Choices = Choices.Collins,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if type is Choices.Collins:
        return collins_cal_vmax(
            impactor, target, energy_blast, altitudeBurst
        )

    return None


def cal_shock_damage(impactor: Impactor, target: Target, opressure: float = None,
                     vmax: float = None, type: Choices = Choices.Collins,) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if type is Choices.Collins:
        return collins_cal_shock_damage(
            impactor, target, opressure, vmax
        )

    return None


def cal_dec_level(
    impactor: Impactor,
    target: Target,
    energy_blast: float = None,
    altitudeBurst: float = None,
    type: Choices = Choices.Collins,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if type is Choices.Collins:
        return collins_cal_dec_level(
            impactor, target, energy_blast, altitudeBurst
        )

    return None


def cal_TsunamiArrivalTime(
    impactor: Impactor,
    target: Target,
    wdiameter: float = None,
    type: Choices = Choices.Collins,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if type is Choices.Collins:
        return collins_cal_TsunamiArrivalTime(impactor, target, wdiameter)
    elif type is Choices.Example:
        return NotImplementedError("Error")

    return None


def cal_WaveAmplitudeUpperLimit(
    impactor: Impactor,
    target: Target,
    wdiameter: float = None,
    type: Choices = Choices.Collins,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if type is Choices.Collins:
        return collins_cal_WaveAmplitudeUpperLimit(
            impactor, target, wdiameter
        )

    return None


def cal_WaveAmplitudeLowerLimit(
    impactor: Impactor,
    target: Target,
    wdiameter: float = None,
    type: Choices = Choices.Collins,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if type is Choices.Collins:
        return collins_cal_WaveAmplitudeLowerLimit(
            impactor, target, wdiameter
        )

    return None



# -*- encoding: utf-8 -*-
"""
Filename         :Targets.py
Description      :
Time             :2022/07/10 09:38:31
Author           :daniel
Version          :1.0
"""


from .checker import numeric_checker, positive_checker


class Target:
    def __init__(
        self,
        depth,
        distance,
        density,
        ttype=3,
        rhoSurface=1,
        dragC=2,
        schaleHeight=8000,
        fp=7,
        pEarth=1.794 * 10 ** 32,
        mEarth=5.97 * 10 ** 24,
        lEarth=5.86 * 10 ** 33,
        g=9.8,
        R_earth=6370,
        surface_wave_v=5,
        melt_coeff=8.9 * 10 ** -21,
        vEarth=1.1 * 10 ** 12,
        Po=10 ** 5,
        seefloor_density=2700,
    ):

        if density == 1000 and depth == 0:
            raise ValueError(
                "density and depth can not be equal to 1000 and 0")

        self.set_density(density=density)
        self.set_rhoSurface(rhoSurface=rhoSurface)
        self.set_dragC(dragC=dragC)
        self.set_fp(fp=fp)
        self.set_schaleHeight(schaleHeight=schaleHeight)
        self.set_pEarth(pEarth=pEarth)
        self.set_mEarth(mEarth=mEarth)
        self.set_lEarth(lEarth=lEarth)
        self.set_g(g=g)
        self.set_R_earth(R_earth=R_earth)
        self.set_surface_wave_v(surface_wave_v=surface_wave_v)
        self.set_melt_coeff(melt_coeff=melt_coeff)
        self.set_vEarth(vEarth=vEarth)
        self.set_depth(depth=depth)
        self.set_distance(distance=distance)
        self.set_Po(Po=Po)
        self.set_seefloor_density(seefloor_density=seefloor_density)
        self.set_ttype(ttype=ttype)

        return

    """setter
    setter function set the value after checker pass
    """

    def set_density(self, density):
        if numeric_checker(density) == False:
            raise ValueError("density must be numeric")
        if positive_checker(density) == False:
            raise ValueError("density must be positive")

        self.density = density

    def set_rhoSurface(self, rhoSurface):
        if numeric_checker(rhoSurface) == False:
            raise ValueError("rhoSurface must be numeric")
        if positive_checker(rhoSurface) == False:
            raise ValueError("rhoSurface must be positive")

        self.rhoSurface = rhoSurface

    def set_dragC(self, dragC):
        if numeric_checker(dragC) == False:
            raise ValueError("dragC must be numeric")
        if positive_checker(dragC) == False:
            raise ValueError("dragC must be positive")

        self.dragC = dragC

    def set_fp(self, fp):
        if numeric_checker(fp) == False:
            raise ValueError("fp must be numeric")
        if positive_checker(fp) == False:
            raise ValueError("fp must be positive")

        self.fp = fp

    def set_schaleHeight(self, schaleHeight):
        if numeric_checker(schaleHeight) == False:
            raise ValueError("schaleHeight must be numeric")
        if positive_checker(schaleHeight) == False:
            raise ValueError("schaleHeight must be positive")

        self.schaleHeight = schaleHeight

    def set_pEarth(self, pEarth):
        if numeric_checker(pEarth) == False:
            raise ValueError("pEarth must be numeric")
        if positive_checker(pEarth) == False:
            raise ValueError("pEarth must be positive")

        self.pEarth = pEarth

    def set_mEarth(self, mEarth):
        if numeric_checker(mEarth) == False:
            raise ValueError("mEarth must be numeric")
        if positive_checker(mEarth) == False:
            raise ValueError("mEarth must be positive")

        self.mEarth = mEarth

    def set_lEarth(self, lEarth):
        if numeric_checker(lEarth) == False:
            raise ValueError("lEarth must be numeric")
        if positive_checker(lEarth) == False:
            raise ValueError("lEarth must be positive")

        self.lEarth = lEarth

    def set_g(self, g):
        if numeric_checker(g) == False:
            raise ValueError("g must be numeric")
        if positive_checker(g) == False:
            raise ValueError("g must be positive")

        self.g = g

    def set_R_earth(self, R_earth):
        if numeric_checker(R_earth) == False:
            raise ValueError("R_earth must be numeric")
        if positive_checker(R_earth) == False:
            raise ValueError("R_earth must be positive")

        self.R_earth = R_earth

    def set_surface_wave_v(self, surface_wave_v):
        if numeric_checker(surface_wave_v) == False:
            raise ValueError("surface_wave_v must be numeric")
        if positive_checker(surface_wave_v) == False:
            raise ValueError("surface_wave_v must be positive")

        self.surface_wave_v = surface_wave_v

    def set_melt_coeff(self, melt_coeff):
        if numeric_checker(melt_coeff) == False:
            raise ValueError("melt_coeff must be numeric")
        if positive_checker(melt_coeff) == False:
            raise ValueError("melt_coeff must be positive")

        self.melt_coeff = melt_coeff

    def set_vEarth(self, vEarth):
        if numeric_checker(vEarth) == False:
            raise ValueError("vEarth must be numeric")
        if positive_checker(vEarth) == False:
            raise ValueError("vEarth must be positive")

        self.vEarth = vEarth

    def set_depth(self, depth):
        if numeric_checker(depth) == False:
            raise ValueError("depth must be numeric")
        if positive_checker(depth) == False:
            raise ValueError("depth must be positive")

        self.depth = depth

    def set_distance(self, distance):
        if numeric_checker(distance) == False:
            raise ValueError("distance must be numeric")
        if positive_checker(distance) == False:
            raise ValueError("distance must be positive")

        self.distance = distance

    def set_Po(self, Po):
        if numeric_checker(Po) == False:
            raise ValueError("Po must be numeric")
        if positive_checker(Po) == False:
            raise ValueError("Po must be positive")

        self.Po = Po

    def set_seefloor_density(self, seefloor_density):
        if numeric_checker(seefloor_density) == False:
            raise ValueError("seefloor_density must be numeric")
        if positive_checker(seefloor_density) == False:
            raise ValueError("seefloor_density must be seefloor_densitysitive")

        self.seefloor_density = seefloor_density

    def set_ttype(self, ttype):
        if numeric_checker(ttype) == False:
            raise ValueError("ttype must be numeric")
        if positive_checker(ttype) == False:
            raise ValueError("ttype must be ttypesitive")

        self.ttype = ttype

    """getter
    getter return the member value without access it directly
    """

    def get_rhoSurface(self):
        return self.rhoSurface

    def get_dragC(self):
        return self.dragC

    def get_schaleHeight(self):
        return self.schaleHeight

    def get_fp(self):
        return self.fp

    def get_pEarth(self):
        return self.pEarth

    def get_mEarth(self):
        return self.mEarth

    def get_lEarth(self):
        return self.lEarth

    def get_g(self):
        return self.g

    def get_R_earth(self):
        return self.R_earth

    def get_surface_wave_v(self):
        return self.surface_wave_v

    def get_melt_coeff(self):
        return self.melt_coeff

    def get_v_earth(self):
        return self.vEarth

    def get_depth(self):
        return self.depth

    def get_distance(self):
        return self.distance

    def get_Po(self):
        return self.Po

    def get_density(self):
        return self.density



try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(name='impactEffects',
      #   use_scm_version=True,
      version="0.0.1",
      setup_requires=['setuptools_scm'],
      description="Environment for MPM assessment 3.",
      long_description="Environment for MPM assessement 3.",
      url="https://github.com/acse-dx121/impact-effects.git",
      author="Imperial College London",
      author_email='rhodri.nelson@imperial.ac.uk',
      packages=['impactEffects'])

with open("README.md", "r") as fh:
    long_description = fh.read()



# -*- encoding: utf-8 -*-
"""
Filename         :config.py
Description      :
Time             :2022/07/10 09:38:07
Author           :daniel
Version          :1.0
"""

import math
from enum import Enum, unique

# PI
PI = math.pi
joules2megatones = 1 / (4.186 * 10 ** 15)

# rename the functions
exp = math.exp
log = math.log
cos = math.cos
sin = math.sin
tan = math.tan
tanh = math.tanh
atan = math.atan
acos = math.acos
sqrt = math.sqrt


@unique
class Choices(Enum):
    Collins = 0
    Example = 256



# -*- encoding: utf-8 -*-
"""
Filename         :core.py
Description      :
Time             :2022/07/10 09:37:51
Author           :daniel
Version          :1.0
"""
import logging

from impactEffects.instances.ImpactorClass import *
from impactEffects.instances.TargetClass import *
from impactEffects.core.config import *


def collins_kinetic_energy(impactor: Impactor) -> float:
    """

    Arguments
    ---------
    impactor: Instance of Impactor, containning

    Returns
    -------

    """

    return impactor.get_energy0()


def collins_kinetic_energy_megatons(impactor: Impactor) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """

    return impactor.get_energy0_megatons()


def collins_rec_time(impactor: Impactor) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """

    mass = impactor.get_mass()

    # Compute the recurrence interval for this energy impact
    # New model (after Bland and Artemieva (2006) MAPS 41 (607-621).
    if mass < 3:
        rec_time = 10 ** (-4.568) * mass ** 0.480
    elif mass < 1.7e10:
        rec_time = 10 ** (-4.739) * mass ** 0.926
    elif mass < 3.3e12:
        rec_time = 10 ** (0.922) * mass ** 0.373
    elif mass < 8.4e14:
        rec_time = 10 ** (-0.086) * mass ** 0.454
    else:
        rec_time = 10 ** (-3.352) * mass ** 0.672

    energy0_megatons = impactor.get_energy0_megatons()
    rec_time = max(rec_time, 110 * (energy0_megatons) ** 0.77)

    return rec_time


def collins_cal_iFactor(impactor: Impactor, target: Target) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """

    (
        pdensity,
        dragC,
        rhoSurface,
        scaleHeight,
        pdiameter,
        velocity,
        theta,
        fp,
    ) = (
        impactor.get_density(),
        target.get_dragC(),
        target.get_rhoSurface(),
        target.get_schaleHeight(),
        impactor.get_pdiameter(),
        impactor.get_velocity(),
        impactor.get_theta(),
        target.get_fp(),
    )

    # Approximate the strength of the impactor using the density function in
    # Eq. 9 of Collins et al. (2005)
    _yield = 10 ** (2.107 + 0.0624 * pdensity ** (1 / 2))
    # Define a relative strength of the impactor compared to the
    # maximum possible stagnation pressure on entry
    _rStrength = _yield / (rhoSurface * (velocity * 1000) ** 2)
    # Define the exponent of Eq. 8 for the case of impat at the surface
    _av = (
        3
        * rhoSurface
        * dragC
        * scaleHeight
        / (4 * pdensity * pdiameter * math.sin(theta * PI / 180))
    )

    collins_iFactor = 2.7185 * _av * _rStrength
    return collins_iFactor, _av, _rStrength


def collins_burst_velocity_at_zero(
    impactor: Impactor, target: Target
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    i_factor, _av, _ = collins_cal_iFactor(impactor, target)

    # check
    if i_factor < 1:
        logging.warning("I_factor should be greater than 1!")
        return None

    # Burst altitude is zero
    altitudeBurst = 0
    # Define the terminal velocity
    # Assuming drag coefficient of 2
    _vTerminal = min(
        impactor.get_velocity(),
        (
            4
            * impactor.density
            * impactor.pdiameter
            * target.g
            / (3 * target.rhoSurface * target.dragC)
        )
        ** (1 / 2),
    )

    # Define the surface velocity assuming continual spreading using Eq. 8
    _vSurface = impactor.vInput * 1000 * math.exp(-_av)

    # Take the maximum of the extrapolated surface velocity
    # and the terminal velocity
    if _vTerminal > _vSurface:
        velocity_at_surface = _vTerminal
    else:
        velocity_at_surface = _vSurface

    return velocity_at_surface/1000


def collins_altitude_of_breakup(
    impactor: Impactor,
    target: Target,
    collins_iFactor: float = None,
    rStrength: float = None,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """

    if collins_iFactor is None or rStrength is None:
        collins_iFactor, av, rStrength = collins_cal_iFactor(
            impactor, target
        )

    # Compute the first term in Eq. 11
    altitude1 = -target.schaleHeight * log(rStrength)

    # Define the second, third and fourth terms (inside the brackets) in Eq. 11
    omega = (
        1.308
        - 0.314 * collins_iFactor
        - 1.303 * (1 - collins_iFactor) ** 0.5
    )

    # Compute the breakup altitude by
    # combining above parameters to evaluate Eq. 11
    altitudeBU = altitude1 - omega * target.schaleHeight

    return altitudeBU


def collins_velocity_at_breakup(
    impactor: Impactor,
    target: Target,
    av: float = None,
    altitudeBU: float = None,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if av is None:
        collins_iFactor, av, rStrength = collins_cal_iFactor(
            impactor, target
        )

    if altitudeBU is None:
        altitudeBU = collins_altitude_of_breakup(
            impactor, target, collins_iFactor
        )

    # m/s
    return (
        impactor.velocity
        * 1000
        * exp(-av * math.exp(-altitudeBU / target.schaleHeight))
    )


def collins_dispersion_length_scale(
    impactor: Impactor, target: Target, altitudeBU: float = None
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if altitudeBU is None:
        altitudeBU = collins_altitude_of_breakup(impactor, target)

    # Assuming drag coefficient of 2
    return (
        impactor.pdiameter
        * sin(impactor.theta * PI / 180)
        * (impactor.density / (target.dragC * target.rhoSurface)) ** 0.5
        * exp(altitudeBU / (2 * target.schaleHeight))
    )


def collins_airburst_altitude(
    impactor: Impactor,
    target: Target,
    alpha2: float = None,
    lDisper: float = None,
    altitudeBU: float = None,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if altitudeBU is None:
        i_factor, _, _rStrength = collins_cal_iFactor(impactor, target)
        altitudeBU = collins_altitude_of_breakup(
            impactor, target, i_factor, _rStrength
        )

    if lDisper is None:
        lDisper = collins_dispersion_length_scale(
            impactor, target, altitudeBU
        )

    if alpha2 is None:
        alpha2 = (target.get_fp() ** 2 - 1) ** (1 / 2)

    # Define the burst altitude using Eq. 18
    altitudePen = (
        2
        * target.get_schaleHeight()
        * log(1 + alpha2 * lDisper / (2 * target.get_schaleHeight()))
    )
    altitudeBurst = altitudeBU - altitudePen
    return altitudeBurst


def collins_brust_velocity(
    impactor: Impactor,
    target: Target,
    altitudeBurst: float = None,
    altitudeBU: float = None,
    vBu: float = None,
    lDisper: float = None,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """

    if altitudeBU is None:
        i_factor, _av, _rStrength = collins_cal_iFactor(impactor, target)
        altitudeBU = collins_altitude_of_breakup(
            impactor, target, i_factor, _rStrength
        )

    if vBu is None:
        i_factor, _av, _rStrength = collins_cal_iFactor(impactor, target)
        vBU = collins_velocity_at_breakup(impactor, target, _av, altitudeBU)

    if lDisper is None:
        lDisper = collins_dispersion_length_scale(
            impactor, target, altitudeBU
        )

    alpha2 = (target.get_fp() ** 2 - 1) ** (1 / 2)
    if altitudeBurst is None:
        altitudeBurst = collins_airburst_altitude(
            impactor, target, alpha2, lDisper, altitudeBU
        )

    # Define factor for evaluating Eq. 17
    vFac = (
        0.75
        * (
            target.get_dragC()
            * target.get_rhoSurface()
            / impactor.get_density()
        )
        ** 0.5
        * exp(-altitudeBU / (2 * target.get_schaleHeight()))
    )  # Assuming drag coefficient of 2

    if altitudeBurst > 0:
        # Evaluate Eq. 19 (without factor lL_0^2 l_disper * pdiameter**2)
        expfac = (
            1
            / 24
            * alpha2
            * (
                24
                + 8 * alpha2 ** 2
                + 6 * alpha2 * lDisper / target.get_schaleHeight()
                + 3 * alpha2 ** 3 * lDisper / target.get_schaleHeight()
            )
        )

        # Evaluate velocity at burst using Eq. 17
        # (note that factor l_disper * pdiameter**2
        # in expfac cancels with same factor in vFac)
        velocity = vBU * exp(-expfac * vFac)
    else:
        # Define (l/H) for use in Eq. 20
        altitudeScale = target.get_schaleHeight() / lDisper

        # Evaluate Eq. 20 (without factor lL_0^2 l_disper * pdiameter**2)
        # (note that this Eq. is not correct in the paper)
        integral = (
            altitudeScale ** 3
            / 3
            * (
                3
                * (4 + 1 / altitudeScale ** 2)
                * exp(altitudeBU / target.get_schaleHeight())
                + 6 * exp(2 * altitudeBU / target.get_schaleHeight())
                - 16 * exp(1.5 * altitudeBU / target.get_schaleHeight())
                - 3 / altitudeScale ** 2
                - 2
            )
        )

        # Evaluate velocity at the surface using Eq. 17
        velocity = vBU * exp(-vFac * integral)

    return velocity / 1000


def collins_dispersion_of_impactor(
    impactor: Impactor,
    target: Target,
    l_disper: float = None,
    altitude_bu: float = None,
    altitude_burst: float = None,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """

    if altitude_bu is None:
        i_factor, _av, _rStrength = collins_cal_iFactor(impactor, target)
        altitude_bu = collins_altitude_of_breakup(
            impactor, target, i_factor, _rStrength
        )

    if l_disper is None:
        l_disper = collins_dispersion_length_scale(
            impactor, target, altitude_bu
        )

    if altitude_burst is None:
        alpha2 = (target.get_fp() ** 2 - 1) ** (1 / 2)
        altitude_burst = collins_airburst_altitude(
            impactor, target, alpha2, l_disper, altitude_bu
        )

    if altitude_burst > 0:
        # raise ValueError("Impactor is not dispersionless at the surface")
        return 0

    dispersion = impactor.get_pdiameter() * (
        1
        + 4
        * (target.get_schaleHeight() / l_disper) ** 2
        * (exp(altitude_bu / (2 * target.get_schaleHeight())) - 1) ** 2
    ) ** (1 / 2)

    return dispersion


def collins_fraction_of_momentum(
    impactor: Impactor, target: Target, velocity: float = None
):
    """

    Arguments
    ---------


    Returns
    -------

    """

    if velocity is None:
        velocity = collins_brust_velocity(impactor, target)

    linmom = impactor.get_mass() * (velocity * 1000)
    angmom = (
        impactor.get_mass()
        * (velocity * 1000)
        * cos(impactor.get_theta() * PI / 180)
        * target.get_R_earth()
    )

    # relativistic effects, multiply energy by 1/sqrt(1 - v^2/c^2)
    if impactor.get_velocity() > (0.25 * 3 * 10 ** 5):
        beta = 1 / (1 - impactor.get_velocity() ** 2 / 9 * 10 ** 10) ** 0.5
        energy0 = impactor.energy0 * beta
        linmom *= beta
        angmom *= beta

    lratio = angmom / target.get_lEarth()
    pratio = linmom / target.get_pEarth()
    return lratio, pratio


def collins_cal_trot_change(
    impactor: Impactor, target: Target, velocity: float = None
):
    """

    Arguments
    ---------


    Returns
    -------

    """

    if velocity is None:
        velocity = collins_brust_velocity(impactor, target)

    mass = impactor.get_mass()
    mEarth = target.get_mEarth()
    theta = impactor.get_theta()
    R_earth = target.get_R_earth()

    return (
        (1.25 / PI)
        * (mass / mEarth)
        * cos(theta * PI / 180)
        / R_earth
        * velocity
        * (24.0 * 60.0 * 60.0) ** 2
    )


def collins_cal_energy_atmosphere(
    impactor: Impactor, target: Target, velocity: float = None
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """

    if velocity is None:
        velocity = collins_brust_velocity(impactor, target)

    # print("velocity: ", velocity, " origin_vel:", impactor.get_velocity())
    energy_atmosphere = (
        0.5
        * impactor.get_mass()
        * ((impactor.get_velocity() * 1000) ** 2 - (velocity * 1000) ** 2)
    )
    return energy_atmosphere


def collins_cal_energy_blast_surface(
    impactor: Impactor,
    target: Target,
    velocity: float = None,
    altitudeBurst: float = None,
    energy_atmosphere: float = None,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if altitudeBurst is None or velocity is None:
        i_factor, _av, _rStrength = collins_cal_iFactor(impactor, target)
        altitudeBU = collins_altitude_of_breakup(
            impactor, target, i_factor, _rStrength
        )
        lDisper = collins_dispersion_length_scale(
            impactor, target, altitudeBU
        )

        alpha2 = (target.get_fp() ** 2 - 1) ** (1 / 2)
        altitudeBurst = collins_airburst_altitude(
            impactor, target, alpha2, lDisper, altitudeBU
        )
        velocity = collins_brust_velocity(
            impactor, target, altitudeBurst, altitudeBU, None, lDisper
        )

    if energy_atmosphere is None:
        energy_atmosphere = collins_cal_energy_atmosphere(
            impactor, target, velocity
        )

    if altitudeBurst > 0:
        # Blast energy is airburst energy (kTons)
        energy_blast = energy_atmosphere / (4.186 * 10 ** 12)
        energy_surface = energy_atmosphere
    else:
        altitudeBurst = 0
        energy_surface = 0.5 * impactor.get_mass() * (velocity * 1000) ** 2
        # Blast energy is larger of airburt and impact energy (kTons)
        if energy_atmosphere > energy_surface:
            energy_blast = energy_atmosphere / (4.186 * 10 ** 12)
        else:
            energy_blast = energy_surface / (4.186 * 10 ** 12)

    return energy_blast, energy_surface


def collins_cal_mass_of_water(impactor: Impactor, target: Target) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """

    mwater = (
        (PI * impactor.get_pdiameter() ** 2 / 4)
        * (target.get_depth() / sin(impactor.get_theta() * PI / 180))
        * 1000
    )
    return mwater


def collins_cal_velocity_projectile(
    impactor: Impactor, target: Target, velocity: float = None
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """

    if velocity is None:
        velocity = collins_brust_velocity(impactor, target)

    vseafloor = velocity * exp(
        -(3 * 1000 * 0.877 * target.get_depth())
        / (
            2
            * impactor.get_density()
            * impactor.get_pdiameter()
            * sin(impactor.get_theta() * PI / 180)
        )
    )
    return vseafloor


def collins_cal_energy_at_seafloor(
    impactor: Impactor, target: Target, vseafloor: float = None
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if vseafloor is None:
        vseafloor = collins_cal_velocity_projectile(impactor, target)

    energy_seafloor = 0.5 * impactor.get_mass() * (vseafloor * 1000) ** 2
    return energy_seafloor


def collins_cal_ePIcentral_angle(target: Target) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """

    return (180 / PI) * (target.get_distance() / target.get_R_earth())


def collins_cal_scaling_diameter_constant(target: Target) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """

    if target.ttype == 1:
        Cd = 1.88
        beta = 0.22
    elif target.ttype == 2:
        Cd = 1.54
        beta = 0.165
    else:
        Cd = 1.6
        beta = 0.22
    return Cd, beta


def collins_cal_anglefac(impactor: Impactor) -> float:
    return (sin(impactor.get_theta() * PI / 180)) ** (1 / 3)


def collins_cal_wdiameter(
    impactor: Impactor,
    target: Target,
    anglefac: float = None,
    velocity: float = None,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if target.depth == 0:
        logging.warning("Target depth is 0")
        exit(0)

    if velocity is None:
        velocity = collins_brust_velocity(impactor, target)

    if anglefac is None:
        anglefac = collins_cal_anglefac(impactor)

    mass, tdensity, g, pdiameter = (
        impactor.get_mass(),
        target.get_density(),
        target.get_g(),
        impactor.get_pdiameter(),
    )

    wdiameter = (
        1.88
        * ((mass / tdensity) ** (1 / 3))
        * ((1.61 * g * pdiameter) / (velocity * 1000) ** 2) ** (-0.22)
    )
    wdiameter *= anglefac

    # update tdensity which should be return as value
    logging.log(
        logging.INFO,
        "update the impactor density for seafloor: %f".format(wdiameter),
    )

    return wdiameter


def collins_cal_transient_crater_diameter(
    impactor: Impactor,
    target: Target,
    Cd: float = None,
    beta: float = None,
    anglefac: float = None,
    vseafloor: float = None,
    dispersion: float = None,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if Cd is None or beta is None:
        Cd, beta = collins_cal_scaling_diameter_constant(target=target)

    if anglefac is None:
        anglefac = collins_cal_anglefac(impactor)

    if vseafloor is None:
        vseafloor = collins_cal_velocity_projectile(impactor, target)

    if dispersion is None:
        dispersion = collins_dispersion_of_impactor(impactor, target)

    mass, g, pdiameter = (
        impactor.get_mass(),
        target.get_g(),
        impactor.get_pdiameter(),
    )

    if target.depth != 0:
        tdensity = target.seefloor_density
    else:
        tdensity = target.density

    # print(
    #     "mass: {, tdensity: {, g: {, pdiameter: {".format(
    #         mass, tdensity, g, pdiameter
    #     )
    # )
    Dtr = (
        Cd
        * ((mass / tdensity) ** (1 / 3))
        * ((1.61 * g * pdiameter) / (vseafloor * 1000) ** 2) ** (-beta)
    )
    Dtr *= anglefac

    if (
        dispersion >= Dtr
    ):  # if crater field is formed, compute crater dimensions assuming
        Dtr /= 2
        # impact of largest fragment (with diameter = 1/2 initial diameter)

    return Dtr


def collins_cal_depthr(
    impactor: Impactor, target: Target, Dtr: float = None
):
    """

    Arguments
    ---------


    Returns
    -------

    """
    if Dtr is None:
        Dtr = collins_cal_transient_crater_diameter(impactor, target)

    return Dtr / 2.828


def collins_cal_cdiamater(
    impactor: Impactor, target: Target, Dtr: float = None
):
    """

    Arguments
    ---------


    Returns
    -------

    """
    if Dtr is None:
        Dtr = collins_cal_transient_crater_diameter(impactor, target)

    if Dtr * 1.25 >= 3200:
        cdiameter = (1.17 * Dtr ** 1.13) / (3200 ** 0.13)
    else:
        cdiameter = 1.25 * Dtr

    return cdiameter


def collins_cal_depthfr(
    impactor: Impactor,
    target: Target,
    Dtr: float = None,
    depthtr: float = None,
    cdiameter: float = None,
):
    """

    Arguments
    ---------


    Returns
    -------

    """
    if Dtr is None:
        Dtr = collins_cal_transient_crater_diameter(impactor, target)
    if depthtr is None:
        depthtr = collins_cal_depthr(impactor, target, Dtr)
    if cdiameter is None:
        cdiameter = collins_cal_cdiamater(impactor, target, Dtr)

    if Dtr * 1.25 >= 3200:
        depthfr = 37 * cdiameter ** 0.301
    else:
        # Breccia lens volume in m^3
        vbreccia = 0.032 * cdiameter ** 3  # in m^3

        # Rim height of final crater in m
        rimHeightf = 0.07 * Dtr ** 4 / cdiameter ** 3

        # Thickness of breccia lens in m
        brecciaThickness = (
            2.8
            * vbreccia
            * ((depthtr + rimHeightf) / (depthtr * cdiameter ** 2))
        )

        # Final crater depth (in m) = transient crater depth +
        # final rim height - breccia thickness
        depthfr = depthtr + rimHeightf - brecciaThickness

    return depthfr


def collins_cal_vCrater(
    impactor: Impactor, target: Target, Dtr: float = None
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if Dtr is None:
        Dtr = collins_cal_transient_crater_diameter(impactor, target)

    return (PI / 24) * (Dtr / 1000) ** 3


def collins_cal_vCrater_vRation(
    impactor: Impactor, target: Target, Dtr: float = None
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if Dtr is None:
        Dtr = collins_cal_transient_crater_diameter(impactor, target)

    vCrater = (PI / 24) * (Dtr / 1000) ** 3
    vRatio = vCrater / target.get_v_earth()
    return vCrater, vRatio


def collins_cal_vMelt(
    impactor: Impactor,
    target: Target,
    velocity: float = None,
    energy_seafloor: float = None,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if velocity is None:
        velocity = collins_brust_velocity(impactor, target)

    if energy_seafloor is None:
        energy_seafloor = collins_cal_energy_at_seafloor(impactor, target)

    if velocity < 12:
        logging.warning("Velocity is less than 12 m/s")
        return None

    vMelt = (
        target.get_melt_coeff()
        * (energy_seafloor)
        * sin(impactor.get_theta() * PI / 180)
    )
    if vMelt > target.vEarth:
        vMelt = target.get_v_earth()

    return vMelt


def collins_cal_mratio_and_mcratio(
    impactor: Impactor,
    target: Target,
    velocity: float = None,
    vMelt: float = None,
    vCrater: float = None,
    Dtr: float = None,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if velocity is None:
        velocity = collins_brust_velocity(impactor, target)

    if velocity < 12:
        logging.warning("Velocity is less than 12 m/s")
        return None, None

    if Dtr is None:
        Dtr = collins_cal_transient_crater_diameter(impactor, target)
    if vMelt is None:
        vMelt = collins_cal_vMelt(impactor, target)
    if vCrater is None:
        vCrater = collins_cal_vCrater(impactor, target, Dtr)

    mcratio = vMelt / vCrater
    mratio = vMelt / target.get_v_earth()
    return mratio, mcratio


def collins_cal_eject_arrival(
    impactor: Impactor, target: Target, altitudeBurst: float = None
):
    """

    Arguments
    ---------


    Returns
    -------

    """
    if altitudeBurst is None:
        i_factor, _av, _rStrength = collins_cal_iFactor(impactor, target)
        altitudeBU = collins_altitude_of_breakup(
            impactor, target, i_factor, _rStrength
        )
        lDisper = collins_dispersion_length_scale(
            impactor, target, altitudeBU
        )

        alpha2 = (target.get_fp() ** 2 - 1) ** (1 / 2)
        altitudeBurst = collins_airburst_altitude(
            impactor, target, alpha2, lDisper, altitudeBU
        )

    if altitudeBurst > 0:
        logging.warning("Altitude of burst is greater than 0")
        return None

    phi = (target.get_distance()) / (2 * target.get_R_earth())
    X = (2 * tan(phi)) / (1 + tan(phi))
    # eccentricity of eliptical path of the ejecta
    e = -((0.5 * (X - 1) ** 2 + 0.5) ** 0.5)
    a = (X * target.get_R_earth() * 1000) / (
        2 * (1 - e ** 2)
    )  # semi major axis of elliptical path

    part1 = (
        a ** 1.5
        / (target.get_g() * (target.get_R_earth() * 1000) ** 2) ** 0.5
    )
    term1 = 2 * atan(((1 - e) / (1 + e)) ** 0.5 * tan(phi / 2))
    term2 = e * (1 - e ** 2) ** 0.5 * sin(phi) / (1 + e * cos(phi))
    ejecta_arrival = 2 * part1 * (term1 - term2)

    return ejecta_arrival


def collins_cal_ejecta_thickness(
    impactor: Impactor,
    target: Target,
    altitudeBurst: float = None,
    Dtr: float = None,
):
    """

    Arguments
    ---------


    Returns
    -------

    """
    if altitudeBurst is None:
        i_factor, _av, _rStrength = collins_cal_iFactor(impactor, target)
        altitudeBU = collins_altitude_of_breakup(
            impactor, target, i_factor, _rStrength
        )
        lDisper = collins_dispersion_length_scale(
            impactor, target, altitudeBU
        )

        alpha2 = (target.get_fp() ** 2 - 1) ** (1 / 2)
        altitudeBurst = collins_airburst_altitude(
            impactor, target, alpha2, lDisper, altitudeBU
        )

    if Dtr is None:
        Dtr = collins_cal_transient_crater_diameter(impactor, target)

    if altitudeBurst > 0:
        logging.warning("Altitude of burst is greater than 0")
        return None

    print("Dtr: ", Dtr, " distance: ", target.distance)
    ejecta_thickness = Dtr ** 4 / (
        112 * (target.get_distance() * 1000) ** 3
    )
    return ejecta_thickness


def collins_cal_d_frag(impactor: Impactor, target: Target, cdiameter: float = None, altitudeBurst: float = None, Dtr: float = None) -> float:

    if altitudeBurst is None:
        i_factor, _av, _rStrength = collins_cal_iFactor(impactor, target)
        altitudeBU = collins_altitude_of_breakup(
            impactor, target, i_factor, _rStrength
        )
        lDisper = collins_dispersion_length_scale(
            impactor, target, altitudeBU
        )

        alpha2 = (target.get_fp() ** 2 - 1) ** (1 / 2)
        altitudeBurst = collins_airburst_altitude(
            impactor, target, alpha2, lDisper, altitudeBU
        )

    if altitudeBurst > 0:
        logging.warning("Altitude of burst is greater than 0")
        exit(1)

    if Dtr is None:
        Dtr = collins_cal_transient_crater_diameter(impactor, target)
    if cdiameter is None:
        cdiameter = collins_cal_cdiamater(impactor, target, Dtr)

    # compute mean fragment size
    a2 = 2.65
    half_diameter = (cdiameter/1000)/2  # half of final crater diameter in km
    dc = 2400 * half_diameter**-1.62

    d_frag = dc*(half_diameter/target.distance)**a2

    return d_frag


def collins_cal_themal(
    impactor: Impactor,
    target: Target,
    energy_surface: float = None,
    altitudeBurst: float = None,
    delta: float = None,
    velocity: float = None,
):
    """

    Arguments
    ---------


    Returns
    -------

    """

    if altitudeBurst is None:
        i_factor, _av, _rStrength = collins_cal_iFactor(impactor, target)
        altitudeBU = collins_altitude_of_breakup(
            impactor, target, i_factor, _rStrength
        )
        lDisper = collins_dispersion_length_scale(
            impactor, target, altitudeBU
        )

        alpha2 = (target.get_fp() ** 2 - 1) ** (1 / 2)
        altitudeBurst = collins_airburst_altitude(
            impactor, target, alpha2, lDisper, altitudeBU
        )

    if velocity is None:
        velocity = collins_brust_velocity(
            impactor,
            target,
            altitudeBurst=altitudeBurst,
            altitudeBU=altitudeBU,
            lDisper=lDisper,
        )

    if energy_surface is None:
        _, energy_surface = collins_cal_energy_blast_surface(
            impactor, target
        )

    if delta is None:
        delta = collins_cal_ePIcentral_angle(target=target)

    if altitudeBurst > 0:
        logging.warning("Altitude of burst is greater than 0")

        return (
            None, None,
            None,
            None,
            None,
            None,
            None,
        )

    energy_megatons = energy_surface / (4.186 * 10 ** 15)
    eta = 3 * 10 ** -3  # factor for scaling thermal energy
    T_star = 3000  # temperature of fireball
    Rf = 2 * 10 ** -6 * (energy_surface) ** (1 / 3)  # Rf is in km
    sigma = 5.67 * 10 ** -8  # Stephan-Boltzmann constant

    thermal_exposure = (eta * energy_surface) / (
        2 * PI * (target.get_distance() * 1000) ** 2
    )

    # h is in km, R_earth is in km
    h = (1 - cos(delta * PI / 180)) * target.get_R_earth()
    Del = acos(h / Rf)
    f = (2 / PI) * (Del - (h / Rf) * sin(Del))

    if h > Rf:
        no_radiation = 1
        return thermal_exposure, no_radiation

    no_radiation = 0
    thermal_exposure *= f

    max_rad_time = Rf / velocity  # Rf in km / velocity in km/s
    irradiation_time = (eta * energy_surface) / (
        2 * PI * (Rf * 1000) ** 2 * sigma * T_star ** 4
    )

    megaton_factor = (energy_megatons) ** (1 / 6)

    thermal_power = log(thermal_exposure) / log(10)
    thermal_power = int(thermal_power)
    thermal_exposure /= 10 ** thermal_power

    return (
        h,
        Rf,
        thermal_exposure,
        no_radiation,
        max_rad_time,
        irradiation_time,
        megaton_factor,
        thermal_power,
    )


def collins_cal_magnitude(
    impactor: Impactor,
    target: Target,
    altitudeBurst: float = None,
    energy_seafloor: float = None,
):
    """

    Arguments
    ---------


    Returns
    -------

    """
    if altitudeBurst is None:
        i_factor, _av, _rStrength = collins_cal_iFactor(impactor, target)
        altitudeBU = collins_altitude_of_breakup(
            impactor, target, i_factor, _rStrength
        )
        lDisper = collins_dispersion_length_scale(
            impactor, target, altitudeBU
        )

        alpha2 = (target.get_fp() ** 2 - 1) ** (1 / 2)
        altitudeBurst = collins_airburst_altitude(
            impactor, target, alpha2, lDisper, altitudeBU
        )

    if energy_seafloor is None:
        energy_seafloor = collins_cal_energy_at_seafloor(impactor, target)

    if altitudeBurst > 0:
        logging.warning("Altitude of burst is greater than 0")
        return None

    magnitude = 0.67 * ((log(energy_seafloor)) / (log(10))) - 5.87
    return magnitude


def collins_cal_magnitude2(
    impactor: Impactor,
    target: Target,
    energy_seafloor: float = None,
    altitudeBurst: float = None,
    delta: float = None,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if altitudeBurst is None:
        i_factor, _av, _rStrength = collins_cal_iFactor(impactor, target)
        altitudeBU = collins_altitude_of_breakup(
            impactor, target, i_factor, _rStrength
        )
        lDisper = collins_dispersion_length_scale(
            impactor, target, altitudeBU
        )

        alpha2 = (target.get_fp() ** 2 - 1) ** (1 / 2)
        altitudeBurst = collins_airburst_altitude(
            impactor, target, alpha2, lDisper, altitudeBU
        )

    if energy_seafloor is None:
        energy_seafloor = collins_cal_energy_at_seafloor(impactor, target)

    if delta is None:
        delta = collins_cal_ePIcentral_angle(target=target)

    if altitudeBurst > 0:
        logging.warning("Altitude of burst is greater than 0")
        return None, None

    Ax = 0  # factor for determining "effective magnitude" at given distance
    magnitude = collins_cal_magnitude(
        impactor,
        target,
        altitudeBurst=altitudeBurst,
        energy_seafloor=energy_seafloor,
    )
    distance = target.distance
    surface_wave_v = target.surface_wave_v

    if distance >= 700:
        Ax = 20 * 10 ** (magnitude - 1.66 * (log(delta) / log(10)) - 3.3)
        Ax /= 1000
    elif distance >= 60:
        Ax = 10 ** (magnitude - (0.0048 * distance + 2.5644))

    else:
        Ax = 10 ** (magnitude - (0.00238 * distance + 1.3342))

    eff_mag = (log(Ax) / log(10)) + 1.4
    seismic_arrival = distance / surface_wave_v

    return eff_mag, seismic_arrival


def collins_cal_shock_arrival(
    impactor: Impactor, target: Target, altitudeBurst: float = None
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if altitudeBurst is None:
        i_factor, _av, _rStrength = collins_cal_iFactor(impactor, target)
        altitudeBU = collins_altitude_of_breakup(
            impactor, target, i_factor, _rStrength
        )
        lDisper = collins_dispersion_length_scale(
            impactor, target, altitudeBU
        )

        alpha2 = (target.get_fp() ** 2 - 1) ** (1 / 2)
        altitudeBurst = collins_airburst_altitude(
            impactor, target, alpha2, lDisper, altitudeBU
        )

    vsound = 330  # speed of sound in m/s
    slantRange = 0  # in km

    # Arrival time is straight line distance divided by sound speed
    # for air burst, distance is slant range from explosion
    slantRange = (
        target.get_distance() ** 2 + (altitudeBurst / 1000) ** 2
    ) ** (1 / 2)
    # distance in meters divided by velocity of sound in m/s
    shock_arrival = (slantRange * 1000) / vsound

    return shock_arrival


def collins_cal_vmax(
    impactor: Impactor,
    target: Target,
    energy_blast: float = None,
    altitudeBurst: float = None,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if altitudeBurst is None:
        i_factor, _av, _rStrength = collins_cal_iFactor(impactor, target)
        altitudeBU = collins_altitude_of_breakup(
            impactor, target, i_factor, _rStrength
        )
        lDisper = collins_dispersion_length_scale(
            impactor, target, altitudeBU
        )

        alpha2 = (target.get_fp() ** 2 - 1) ** (1 / 2)
        altitudeBurst = collins_airburst_altitude(
            impactor, target, alpha2, lDisper, altitudeBU
        )

    if energy_blast is None:
        energy_blast, _ = collins_cal_energy_blast_surface(
            impactor, target, altitudeBurst=altitudeBurst
        )

    Po = target.get_Po()
    vsound = 330  # speed of sound in m/s
    r_cross = 0
    # radius at which relationship between overpressure and distance changes
    # radius at which relationship between
    # overpressure and distance changes (for surface burst)
    r_cross0 = 290
    op_cross = 75000  # overpressure at crossover
    energy_ktons = 0  # energy in kilotons
    d_scale = 0  # distance scaled for 1 kTon blast
    d_smooth = 0
    p_machT = 0
    p_regT = 0

    # energy_ktons = 1000 * energy_megatons
    energy_ktons = energy_blast

    # Scale distance to equivalent for a kiloton explosion
    sf = (energy_ktons) ** (1 / 3)
    d_scale = (target.get_distance() * 1000) / sf

    # Scale burst altitude to equivalent for a kiloton explosion
    z_scale = altitudeBurst / sf
    r_cross = r_cross0 + 0.65 * z_scale
    r_mach = 550 * z_scale / (1.2 * (550 - z_scale))
    if z_scale >= 550:
        r_mach = 1e30

    if altitudeBurst > 0:
        d_smooth = z_scale ** 2 * 0.00328
        p_machT = (
            ((r_cross * op_cross) / 4)
            * (1 / (r_mach + d_smooth))
            * (1 + 3 * (r_cross / (r_mach + d_smooth)) ** (1.3))
        )
        p_regT = 3.14e11 * ((r_mach - d_smooth) ** 2 + z_scale ** 2) ** (
            -1.3
        ) + 1.8e7 * ((r_mach - d_smooth) ** 2 + z_scale ** 2) ** (-0.565)
    else:
        d_smooth = 0
        p_machT = 0

    if d_scale >= (r_mach + d_smooth):
        opressure = (
            ((r_cross * op_cross) / 4)
            * (1 / d_scale)
            * (1 + 3 * (r_cross / d_scale) ** (1.3))
        )
    elif d_scale <= (r_mach - d_smooth):
        opressure = 3.14e11 * (d_scale ** 2 + z_scale ** 2) ** (
            -1.3
        ) + 1.8e7 * (d_scale ** 2 + z_scale ** 2) ** (-0.565)
    else:
        opressure = (
            p_regT
            - (d_scale - r_mach + d_smooth)
            * 0.5
            * (p_regT - p_machT)
            / d_smooth
        )

    # Wind velocity
    vmax = ((5 * opressure) / (7 * Po)) * (
        vsound / (1 + (6 * opressure) / (7 * Po)) ** (1 / 2)
    )

    return vmax, opressure


def collins_cal_shock_damage(impactor: Impactor, target: Target, opressure: float = None, vmax: float = None) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """

    if opressure == None or vmax == None:
        vmax, opressure = collins_cal_vmax(impactor=impactor, target=target)

    # damage descriptions:  structures
    shock_damage = ""
    if opressure >= 42600:
        shock_damage += "Multistory wall-bearing buildings will collapse.\n"
    elif opressure >= 38500:
        shock_damage += " Multistory wall-bearing buildings will experience severe cracking and interior partitions will be blown down\n"

    if opressure >= 26800:
        shock_damage += " Wood frame buildings will almost completely collapse.\n"
    elif opressure >= 22900:
        shock_damage += " Interior partitions of wood frame buildings will be blown down.  Roof will be severely damaged.\n"

    if opressure >= 273000:
        shock_damage += " Multistory steel-framed office-type buildings will suffer extreme frame distortion, incipient collapse.\n"

    if opressure >= 121000:
        shock_damage += " Highway truss bridges will collapse.\n"
    elif opressure >= 100000:
        shock_damage += " Highway truss bridges will suffer substantial distortion of bracing.\n"

    if opressure >= 379000:
        shock_damage += " Highway girder bridges will collapse.\n"

    # damage descriptions:  glass, transportation, forrests
    if opressure >= 6900:
        shock_damage += " Glass windows will shatter.\n"
    elif opressure >= 690:
        shock_damage += " Glass windows may shatter.\n"

    if opressure >= 426000:
        shock_damage += " Cars and trucks will be largely displaced and grossly distorted and will require rebuilding before use.\n"
    elif opressure >= 297000:
        shock_damage += " Cars and trucks will be overturned and displaced, requiring major repairs.\n"

    if vmax >= 62:
        shock_damage += " Up to 90 percent of trees blown down remainder stripped of branches and leaves.\n"
    elif vmax >= 40:
        shock_damage += " About 30 percent of trees blown down remainder have some branches and leaves blown off.\n"

    return shock_damage


def collins_cal_dec_level(
    impactor: Impactor,
    target: Target,
    energy_blast: float = None,
    altitudeBurst: float = None,
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if altitudeBurst is None:
        i_factor, _av, _rStrength = collins_cal_iFactor(impactor, target)
        altitudeBU = collins_altitude_of_breakup(
            impactor, target, i_factor, _rStrength
        )
        lDisper = collins_dispersion_length_scale(
            impactor, target, altitudeBU
        )

        alpha2 = (target.get_fp() ** 2 - 1) ** (1 / 2)
        altitudeBurst = collins_airburst_altitude(
            impactor, target, alpha2, lDisper, altitudeBU
        )

    if energy_blast is None:
        energy_blast, _ = collins_cal_energy_blast_surface(
            impactor, target, altitudeBurst=altitudeBurst
        )

    r_cross = 0
    # radius at which relationship between overpressure and distance changes
    # radius at which relationship between
    # overpressure and distance changes (for surface burst)
    r_cross0 = 290
    op_cross = 75000  # overpressure at crossover
    energy_ktons = 0  # energy in kilotons
    d_scale = 0  # distance scaled for 1 kTon blast
    d_smooth = 0
    p_machT = 0
    p_regT = 0

    # energy_ktons = 1000 * energy_megatons
    energy_ktons = energy_blast

    # Scale distance to equivalent for a kiloton explosion
    sf = (energy_ktons) ** (1 / 3)
    d_scale = (target.get_distance() * 1000) / sf

    # Scale burst altitude to equivalent for a kiloton explosion
    z_scale = altitudeBurst / sf
    r_cross = r_cross0 + 0.65 * z_scale
    r_mach = 550 * z_scale / (1.2 * (550 - z_scale))
    if z_scale >= 550:
        r_mach = 1e30

    if altitudeBurst > 0:
        d_smooth = z_scale ** 2 * 0.00328
        p_machT = (
            ((r_cross * op_cross) / 4)
            * (1 / (r_mach + d_smooth))
            * (1 + 3 * (r_cross / (r_mach + d_smooth)) ** (1.3))
        )
        p_regT = 3.14e11 * ((r_mach - d_smooth) ** 2 + z_scale ** 2) ** (
            -1.3
        ) + 1.8e7 * ((r_mach - d_smooth) ** 2 + z_scale ** 2) ** (-0.565)
    else:
        d_smooth = 0
        p_machT = 0

    if d_scale >= (r_mach + d_smooth):
        opressure = (
            ((r_cross * op_cross) / 4)
            * (1 / d_scale)
            * (1 + 3 * (r_cross / d_scale) ** (1.3))
        )
    elif d_scale <= (r_mach - d_smooth):
        opressure = 3.14e11 * (d_scale ** 2 + z_scale ** 2) ** (
            -1.3
        ) + 1.8e7 * (d_scale ** 2 + z_scale ** 2) ** (-0.565)
    else:
        opressure = (
            p_regT
            - (d_scale - r_mach + d_smooth)
            * 0.5
            * (p_regT - p_machT)
            / d_smooth
        )

    # sound intensity
    if opressure > 0:
        dec_level = 20 * (log(opressure) / log(10))
    else:
        dec_level = 0

    return dec_level


def collins_cal_TsunamiArrivalTime(
    impactor: Impactor, target: Target, wdiameter: float = None
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if wdiameter is None:
        wdiameter = collins_cal_wdiameter(impactor, target)

    TsunamiSpeed = 0  # Tsunami speed in m/s
    TsunamiWavelength = 0  # Tsunami wavelength in m

    # Tsunami arrival time assumes linear wave theory
    TsunamiWavelength = 2.0 * wdiameter
    # print("TsunamiWavelength: ", TsunamiWavelength)
    TsunamiSpeed = sqrt(
        0.5
        * target.get_g()
        * TsunamiWavelength
        / PI
        * tanh(2.0 * PI * target.get_depth() / TsunamiWavelength)
    )
    # print("TsunamiSpeed: ", TsunamiSpeed)
    TsunamiArrivalTime = target.get_distance() * 1000 / TsunamiSpeed

    return TsunamiArrivalTime


def collins_cal_WaveAmplitudeUpperLimit(
    impactor: Impactor, target: Target, wdiameter: float = None
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if wdiameter is None:
        wdiameter = collins_cal_wdiameter(impactor, target)

    MaxWaveAmplitude = 0  # Maximum rim wave amplitude
    MaxWaveRadius = (
        0  # Radius where max rim wave is formed (upper estimate)
    )
    RimWaveExponent = 0  # Attenuation factor for rim wave

    # Define parameters
    RimWaveExponent = 1.0
    MaxWaveRadius = 0.001 * wdiameter

    MaxWaveAmplitude = min(0.07 * wdiameter, target.get_depth())
    WaveAmplitudeUpperLimit = (
        MaxWaveAmplitude
        * (MaxWaveRadius / target.get_distance()) ** RimWaveExponent
    )

    return WaveAmplitudeUpperLimit


def collins_cal_WaveAmplitudeLowerLimit(
    impactor: Impactor, target: Target, wdiameter: float = None
) -> float:
    """

    Arguments
    ---------


    Returns
    -------

    """
    if wdiameter is None:
        wdiameter = collins_cal_wdiameter(impactor, target)

    shallowness = 0  # Ratio of Impactor diameter to water depth
    MaxWaveAmplitude = 0  # Maximum rim wave amplitude
    MinWaveRadius = (
        0  # Radius where max rim wave is formed (lower estimate)
    )
    CollapseWaveRadius = 0  # Radius where collapse wave is formed
    RimWaveExponent = 0  # Attenuation factor for rim wave
    CollapseWaveExponent = 0  # Attenuation factor for collapse wave
    MaxCollapseWaveAmplitude = 0  # Maximum collapse wave amplitude
    CollapseWaveAmplitude = (
        0  # Amplitude of collapse wave at specified distance
    )

    # Define parameters
    shallowness = impactor.get_pdiameter() / target.get_depth()
    RimWaveExponent = 1.0
    MinWaveRadius = 0.0005 * wdiameter

    MaxWaveAmplitude = min(0.07 * wdiameter, target.get_depth())
    WaveAmplitudeLowerLimit = (
        MaxWaveAmplitude
        * (MinWaveRadius / target.get_distance()) ** RimWaveExponent
    )

    # Collapse wave correction to lower limit for deep-water impacts
    if shallowness < 0.5:
        CollapseWaveExponent = 3.0 * exp(-0.8 * shallowness)
        CollapseWaveRadius = 0.0025 * wdiameter
        MaxCollapseWaveAmplitude = 0.06 * min(
            wdiameter / 2.828, target.get_depth()
        )
        CollapseWaveAmplitude = (
            MaxCollapseWaveAmplitude
            * (CollapseWaveRadius / target.get_distance())
            ** CollapseWaveExponent
        )
        WaveAmplitudeLowerLimit = min(
            CollapseWaveAmplitude, WaveAmplitudeLowerLimit
        )

    return WaveAmplitudeLowerLimit
